// @flow
import React, { Component } from 'react'
import PropTypes from 'prop-types'
import { View, TextInput, Text, TouchableOpacity, FlatList } from 'react-native'
import StyleSheet from 'react-native-extended-stylesheet'
import Token from '../../components/Token'
import UserBadge from '../../components/UserBadge'
import { SCREEN_HEIGHT } from '../../utils/device'

const styles = StyleSheet.create({
  wrapper: {
    position: 'relative',
    overflow: 'visible',
    flexDirection: 'row',
    alignItems: 'center',
    zIndex: 99,
    width: '100%',
    height: 80
  },
  autocompleteContainer: {
    flex: 1,
    left: 0,
    position: 'absolute',
    right: 0,
    top: 0,
    zIndex: 1
  },
  outerWrapper: {
    backgroundColor: '#f8f8f8',
    paddingBottom: 10,
    borderBottomColor: '#c8c8c8',
    borderBottomWidth: 1
  },
  innerWrapper: {
    marginLeft: 2
  },
  inputWrapper: {
    flexDirection: 'row',
    flex: 1,
    paddingTop: 1,
    paddingRight: 4,
    paddingBottom: 1,
    paddingLeft: 4,
    overflow: 'hidden',
    flexWrap: 'wrap',
    minHeight: 26,
    alignItems: 'center'
  },
  input: {
    flex: 1,
    height: 25,
    padding: 0,
    margin: 0,
    borderWidth: 0,
    minWidth: 50,
    fontSize: 14
  },
  list: {
    flex: 1,
    borderRadius: 2
  },
  listContainer: {
    position: 'absolute',
    top: 0,
    left: 3,
    right: 3,
    zIndex: 2,
    backgroundColor: '#ffffff',
    borderBottomLeftRadius: 2,
    borderBottomRightRadius: 2,
    shadowColor: '#000000',
    shadowOffset: {
      width: 2,
      height: 4
    },
    shadowOpacity: 0.2
  },
  entry: {
    flexDirection: 'row',
    height: 32,
    paddingLeft: 3,
    alignItems: 'center'
  },
  entryText: {
    fontSize: 13,
    flex: 1
  },
  entryTextWrapper: {
    flex: 1,
    marginLeft: 5,
    height: 30,
    alignItems: 'center',
    flexDirection: 'row',
    marginRight: 5
  },
  textTo: {
    marginLeft: 5,
    color: '#a8a8a8'
  }
})

export default class TokenAutocomplete extends Component {
  static propTypes = {
    addToken: PropTypes.func.isRequired,
    options: PropTypes.arrayOf(PropTypes.string).isRequired,
    removeToken: PropTypes.func.isRequired,
    values: PropTypes.arrayOf(PropTypes.string).isRequired
  }

  constructor (props) {
    super(props)
    this.state = {
      search: ''
    }
    this.focus = this.focus.bind(this)
    this.keyPress = this.keyPress.bind(this)
    this.renderAutocompleteEntry = this.renderAutocompleteEntry.bind(this)
    this.measureHeight = this.measureHeight.bind(this)
  }

  focus () {
    this.textInput.focus()
  }

  keyPress ({ nativeEvent }) {
    const { key } = nativeEvent
    const { search } = this.state
    if (key === 'Enter') {
      this.addToken(search)
    }
  }

  measureHeight ({ nativeEvent }) {
    const { layout } = nativeEvent
    const { height } = layout
    this.setState({ height })
  }

  addToken (name) {
    this.textInput.blur()
    this.props.addToken(name)
    this.setState({ search: '' })
  }

  renderAutocompleteEntry (rowData) {
    const { item } = rowData
    const { name, isLast } = item
    const borderStyle = {
      borderBottomWidth: 1,
      borderColor: '#c8c8c8'
    }
    return (
      <TouchableOpacity
        style={styles.entry}
        onPress={() => this.addToken(name)}
      >
        <UserBadge size={22} name={name} />
        <View style={[styles.entryTextWrapper, !isLast && borderStyle]}>
          <Text style={styles.entryText}>{name}</Text>
        </View>
      </TouchableOpacity>
    )
  }

  renderAutocomplete () {
    const { options } = this.props
    const { search, height } = this.state
    const listPosition = {
      top: height,
      maxHeight: SCREEN_HEIGHT - height
    }
    if (!search || !search.length) {
      return <View style={[styles.listContainer, listPosition]} />
    }

    const filteredOptions = options
      .filter(
        option => option.toLowerCase().indexOf(search.toLowerCase()) !== -1
      )
      .map(option => ({ name: option, isLast: false }))
    if (!filteredOptions.length) {
      filteredOptions.unshift({ name: search, isLast: false })
    }
    filteredOptions[filteredOptions.length - 1].isLast = true
    return (
      <View style={[styles.listContainer, listPosition]}>
        <FlatList
          data={filteredOptions}
          style={styles.list}
          renderItem={this.renderAutocompleteEntry}
          keyExtractor={(_, index) => index.toString()}
        />
      </View>
    )
  }

  render () {
    const { values, removeToken } = this.props
    return (
      <View style={styles.autocompleteContainer}>
        <View
          style={[styles.wrapper, styles.outerWrapper]}
          onLayout={this.measureHeight}
        >
          <Text style={styles.textTo}>To:</Text>
          <View style={[styles.wrapper, styles.innerWrapper]}>
            <TouchableOpacity style={styles.inputWrapper} onPress={this.focus}>
              {values.map((value, index) => (
                <Token
                  key={index}
                  index={index}
                  value={value}
                  remove={removeToken}
                />
              ))}
              <TextInput
                ref={ref => (this.textInput = ref)}
                value={this.state.search}
                onKeyPress={this.keyPress}
                onChangeText={search => this.setState({ search })}
                style={styles.input}
                underlineColorAndroid='rgba(0,0,0,0)'
              />
            </TouchableOpacity>
          </View>
          <View style={styles.autocompleteContainer}>
            {this.renderAutocomplete()}
          </View>
        </View>
      </View>
    )
  }
}
