import { Record, Map } from 'immutable'
import { ResourcesState } from '../reducer'
import * as utils from '../utils'

const TestRecord = Record({
  id: null,
  value: null
})

function entityToTestRecord (entity) {
  return TestRecord({
    id: entity.id,
    value: parseInt(entity.id, 10)
  })
}

describe('Resource Utils', () => {
  describe('listToMap', () => {
    it('converts the list to a map of records', () => {
      const list = ['0', '1']
      const resources = {
        0: { id: '0' },
        1: { id: '1' },
        2: { id: '2' }
      }

      const result = utils.listToMap(resources, entityToTestRecord, list)
      expect(Object.keys(result)).toHaveLength(list.length)
      list.forEach((item, idx) => {
        expect(result[item].id).toBe(item)
        expect(result[item].value).toBe(idx)
      })
    })
  })

  describe('createEntitiesToMap', () => {
    const entityToMap = utils.createEntitiesToMap('Test', entityToTestRecord)

    it('takes list items from entities[entityName]', () => {
      const list = ['0', '1']
      const entities = {
        Test: {
          0: { id: '0' },
          1: { id: '1' },
          2: { id: '2' }
        }
      }

      const result = entityToMap(entities, list)
      expect(Object.keys(result)).toHaveLength(list.length)
      list.forEach((item, idx) => {
        expect(result[item].id).toBe(item)
        expect(result[item].value).toBe(idx)
      })
    })

    it('works without list and missing [entityName] in entities object', () => {
      const result = entityToMap({})
      expect(Object.keys(result)).toHaveLength(0)
    })
  })

  describe('createResourceSelector', () => {
    it('returns expected resource map', () => {
      const resources = ResourcesState()
      const databases = utils.createResourceSelector('databases')
      expect(databases({ resources })).toBe(resources.get('databases'))
    })
  })

  describe('entityToPatchedRecord', () => {
    const FooBarRecord = Record({
      id: undefined,
      foo: undefined,
      bar: undefined
    })
    const entityToRecord = entity => new FooBarRecord(entity)
    const initialRecord = entityToRecord({
      id: '1',
      foo: 'foo-1',
      bar: 'bar-1'
    })
    const records = Map({
      1: initialRecord
    })

    it('should return a new (partial) Record if no previous matching Record exist', () => {
      const record = utils.entityToPatchedRecord(
        records,
        {
          id: '2',
          foo: 'foo-2'
        },
        entityToRecord
      )
      expect(record.toJS()).toEqual({
        id: '2',
        foo: 'foo-2',
        bar: undefined
      })
    })

    it('should return a updated (patched) Record for a partial entity', () => {
      const patchedRecord = utils.entityToPatchedRecord(
        records,
        {
          id: '1',
          foo: 'foo-2'
        },
        entityToRecord
      )
      expect(patchedRecord.toJS()).toEqual({
        id: '1',
        foo: 'foo-2',
        bar: 'bar-1'
      })
    })

    it('should return a updated (patched) Record for a full entity', () => {
      const patchedRecord = utils.entityToPatchedRecord(
        records,
        {
          id: '1',
          foo: 'foo-2',
          bar: 'bar-2'
        },
        entityToRecord
      )
      expect(patchedRecord.toJS()).toEqual({
        id: '1',
        foo: 'foo-2',
        bar: 'bar-2'
      })
    })
  })
})
