// @flow
import humps from 'humps'
import uuid from 'utils/generateuuid'
import CONSTANTS, { MESSAGE_TYPE } from './constants'
import { UserSelectors } from 'resources/user'
import * as ConversationApi from './api'
import { convertMessage } from './utils'
import { upload } from 'utils/upload'
import { post } from 'utils/http'
import { API_URL } from '../../config'

export function loadConversation ({ conversationId }) {
  return async (dispatch, getState) => {
    const userId = UserSelectors.currentUserId(getState())
    try {
      const messages = await ConversationApi.fetchMessages({
        conversationId,
        userId
      })
      const conversation = await ConversationApi.fetchConversationInfo({
        conversationId
      })
      dispatch({
        type: CONSTANTS.FETCH_INFO,
        payload: {
          id: conversationId,
          messages: messages.data
            .map(convertMessage)
            .sort((a, b) => +new Date(b.createdAt) - +new Date(a.createdAt)),
          partecipants: Object.values(conversation[0])
        }
      })
    } catch (e) {
      console.error('Conversation error', e)
    }
  }
}

export function sendMessage ({
  conversationId,
  text = '',
  mediaUrl = null,
  type = MESSAGE_TYPE.APP
}) {
  return async (dispatch, getState) => {
    try {
      const userId = UserSelectors.currentUserId(getState())
      const senderName = UserSelectors.currentUser(getState()).get('name')
      const messageId = uuid()
      dispatch({
        type: CONSTANTS.SEND_MESSAGE,
        payload: {
          id: messageId,
          conversationId,
          message: text,
          sendingFrom: userId,
          senderName,
          userId: userId,
          type
        }
      })
      await ConversationApi.sendMessage({
        conversationId,
        message: text,
        sendingFrom: userId,
        userId: userId,
        type
      })
    } catch (error) {
      console.log('Conversation::sendMessage', error)
    }
  }
}

export function receivedSocketMessage ({
  conversationId,
  text,
  mediaUrl,
  id,
  ...rest
}) {
  return (dispatch, getState) => {
    dispatch({
      type: CONSTANTS.RECEIVE_MESSAGE,
      payload: {
        currentUserId: UserSelectors.currentUserId(getState()),
        conversationId,
        text,
        // socket seems to return mediaUrl as text?
        mediaUrl: mediaUrl,
        id,
        ...rest
      }
    })
  }
}

export function uploadMessageImage ({
  conversationId,
  projectId,
  projectLat,
  projectlong,
  projectLocation,
  projectName,
  filename,
  filepath
}) {
  return async (dispatch, getState) => {
    try {
      const userId = UserSelectors.currentUserId(getState())
      const senderName = UserSelectors.currentUser(getState()).get('name')
      const messageId = uuid()
      dispatch({
        type: CONSTANTS.SEND_MESSAGE,
        payload: {
          id: messageId,
          conversationId,
          message: '',
          sendingFrom: userId,
          senderName,
          userId: userId,
          type: MESSAGE_TYPE.APP,
          mediaUrl: filepath
        }
      })
      const data = humps.decamelizeKeys({
        projectId,
        // lat: projectLat || 0.01,
        // long: projectlong || 0.01,
        location: projectLocation,
        name: projectName,
        filename
      })
      const file = [{ name: filename, filename, filepath }]
      const response = await upload(
        `${API_URL}/message/upload/file`,
        data,
        file
      )
      await ConversationApi.sendMessage({
        conversationId,
        message: '',
        sendingFrom: userId,
        userId: userId,
        type: MESSAGE_TYPE.APP,
        mediaUrl: response.mediaUrl
      })
    } catch (error) {
      console.log('Conversation::uploadImage', error)
    }
  }
}

export function createConversationWith ({ projectId, message, partecipants }) {
  return async (dispatch, getState) => {
    const userId = UserSelectors.currentUserId(getState())
    return ConversationApi.createConversationWith({
      projectId,
      sendingFrom: userId,
      partecipants,
      message
    })
  }
}

export function uploadMessageAudio ({
  conversationId,
  duration,
  projectId,
  projectName,
  filename,
  filepath
}) {
  return async (dispatch, getState) => {
    try {
      const userId = UserSelectors.currentUserId(getState())
      const senderName = UserSelectors.currentUser(getState()).get('name')
      const messageId = uuid()
      dispatch({
        type: CONSTANTS.SEND_MESSAGE,
        payload: {
          id: messageId,
          conversationId,
          message: '',
          sendingFrom: userId,
          senderName,
          userId: userId,
          type: MESSAGE_TYPE.AUDIO,
          mediaUrl: filepath
        }
      })
      const file = [{ name: filename, filename, filepath }]
      const name = `audiomessage${Math.ceil(Math.random() * 1000)}`
      const preUploadResponse = await post('/media/upload_audio', {
        userId,
        name,
        duration,
        filename: 'someshortorlongaudiomessage.caf'
      })
      const data = humps.decamelizeKeys({
        name,
        realFilename: filename,
        userId,
        duration,
        filename: preUploadResponse.data.filename
      })
      const response = await upload(
        `${API_URL}/media/upload_audio`,
        data,
        file,
        false
      )
      await ConversationApi.sendMessage({
        conversationId,
        message: '',
        sendingFrom: userId,
        userId: userId,
        type: MESSAGE_TYPE.AUDIO,
        mediaUrl: response.mediaUrl
      })
    } catch (error) {
      console.log('Conversation::uploadMessageAudio', error.data || error)
      throw error
    }
  }
}

export function createSubConversation ({ projectId, subUserId }) {
  return async (dispatch, getState) => {
    const userId = UserSelectors.currentUserId(getState())
    return ConversationApi.createSubConversation({
      projectId,
      userId,
      subUserId
    })
  }
}

export function addToConversation ({ conversationId, conversationUserId }) {
  return async (dispatch, getState) => {
    const userId = UserSelectors.currentUserId(getState())
    return ConversationApi.addToConversation({
      conversationId,
      userId,
      conversationUserId
    })
  }
}
