// @flow
import moment from 'moment-timezone'
import CONSTANTS, { MESSAGE_TYPE } from './constants'
import ConversationSchema from './schema'
import { fromJS } from 'immutable'
import { normalize } from 'normalizr'
import schema from 'resources/schema'
import {
  createEntitiesToMap,
  createPatchedEntitiesToMap
} from 'resources/utils'
import { UserReducers } from 'resources/user'

export function entityToRecord (entity) {
  const { id, messages, partecipants } = entity
  return new ConversationSchema({
    id,
    messages: fromJS(messages),
    partecipants: fromJS(partecipants),
    project: messages && messages[0].project
  })
}

function isSameMessage (msgA, msgB) {
  if (msgA.get('type') !== msgB.get('type')) {
    return false
  }
  if (msgA.get('id') && msgB.get('id')) {
    return msgA.get('id') === msgB.get('id')
  }
  if (
    msgA.get('type') === MESSAGE_TYPE.APP ||
    msgA.get('type') === MESSAGE_TYPE.TEXT
  ) {
    return msgA.get('message') === msgB.get('message')
  }
  return false
}

function removeDuplicates (messages) {
  return messages.filter((message, idx) => {
    return (
      idx === messages.findIndex(message1 => isSameMessage(message, message1))
    )
  })
}

export const entitiesToMap = createEntitiesToMap(
  'conversations',
  entityToRecord
)
export const partialEntitiesToMap = createPatchedEntitiesToMap(
  'conversations',
  entityToRecord
)

export default {
  [CONSTANTS.FETCH_INFO] (state, { payload }) {
    const { entities } = normalize(payload, schema.conversation)
    const newConversations = entitiesToMap(entities)
    const conversations = state.conversations.merge(newConversations)
    const _conversations = state._conversations.setIn(
      ['hasLoaded', payload.id],
      true
    )
    const users = state.users.mergeDeep(
      UserReducers.partialEntitiesToMap(state.users, entities)
    )
    return state.merge({
      conversations,
      _conversations,
      users
    })
  },
  [CONSTANTS.SEND_MESSAGE] (state, { payload }) {
    const {
      id,
      conversationId,
      message,
      sendingFrom,
      senderName,
      type,
      mediaUrl
    } = payload
    const DeviceInfo = require('react-native-device-info')
    const msgFn = DeviceInfo.isEmulator() ? 'unshift' : 'push'
    const oldMessages = state.conversations.getIn([conversationId, 'messages'])
    const newMessages = oldMessages[msgFn](
      fromJS({
        message,
        name: senderName,
        user: sendingFrom,
        contactPerson: false,
        createdAt: moment(new Date()).format('YYYY-MM-DD HH:mm:ss'),
        id,
        type,
        mediaUrl,
        mediaType: null,
        mediaThumbnail: null,
        bidId: null,
        isFile: 0,
        project: state.conversations.getIn([
          conversationId,
          'messages',
          0,
          'project'
        ]),
        isSending: true
      })
    )
    const conversations = state.conversations.mergeIn(
      [conversationId, 'messages'],
      newMessages
    )
    return state.merge({ conversations })
  },
  [CONSTANTS.RECEIVE_MESSAGE] (state, { payload }) {
    /*
    TODO: Fix me - images in socket don't have matching filenames/mediaurl due to s3 upload
    TODO: Fix me - this makes us unablet to detect which message is being sent and which one is sent
    TODO: Fix me - for now we can safely assume that all messages are sent with this
    const { conversationId, text, mediaUrl, id } = payload
    const sendingMessageIndex = state.conversations
      .getIn([conversationId, 'messages'])
      .findIndex(message => {
        const sameText = text && message.get('message') === text
        const sameImage = message.get('mediaUrl')
        return (
          (sameText || sameImage || crossImageText) && message.get('isSending')
        )
      })
    if (sendingMessageIndex === -1) {
      return state
    }
    const conversations = state.conversations
      .setIn(
        [conversationId, 'messages', sendingMessageIndex, 'isSending'],
        false
      )
      .setIn([conversationId, 'messages', sendingMessageIndex, 'id'], id)
    */
    const DeviceInfo = require('react-native-device-info')
    const { conversationId, user, userId, ...rest } = payload
    const msgFn = DeviceInfo.isEmulator() ? 'unshift' : 'push'
    let messages = state.conversations
      .getIn([conversationId, 'messages'])
      .filter(message => !message.get('isSending'))
    messages = messages[msgFn](
      fromJS({
        ...rest,
        createdAt: moment(new Date()).format('YYYY-MM-DD HH:mm:ss'),
        user: userId || user,
        conversationId
      })
    )
    const conversations = state.conversations.setIn(
      [conversationId, 'messages'],
      removeDuplicates(messages)
    )

    return state.merge({ conversations })
  }
}
