import createStoreForTesting from 'test/createstore'
import projectListRecord from 'test/records/projectlist.json'
import projectInfoRecord from 'test/records/projectinfo.json'
import { UserActions } from 'resources/user'
import { ContactSelectors } from 'resources/contact'
import { API_URL } from '../../../config'
import * as actions from '../actions'
import * as selectors from '../selectors'

describe('Project Resource', () => {
  describe('fetchList', () => {
    it('should do the correct api call', async () => {
      const store = createStoreForTesting()
      const userId = '25'
      store.dispatch(UserActions.setCurrentUser(userId))
      store.addResponse(projectListRecord)
      await store.dispatch(actions.fetchList())
      expect(fetch).toHaveBeenCalledWith(`${API_URL}/projects/${userId}`, {
        headers: {
          Accept: 'application/json',
          'Content-Type': 'application/json'
        },
        method: 'GET'
      })
    })
    it('should load a list of projects', async () => {
      const store = createStoreForTesting()
      const userId = '25'
      const projectId = '593'
      const projectSelector = selectors.getProject(() => projectId)
      store.dispatch(UserActions.setCurrentUser(userId))
      store.addResponse(projectListRecord)
      await store.dispatch(actions.fetchList())
      expect(selectors.projectsList(store.getState()).toJS()).toEqual([
        '593',
        '592',
        '591',
        '590'
      ])
      const project = projectSelector(store.getState())
      expect(project.toJS()).toEqual({
        contacts: [],
        id: '593',
        isSub: false,
        long: -80.380851,
        lat: 25.670614,
        link: '',
        location: '',
        name: 'Stubbs water claim',
        owner: '768',
        phoneNumber: '223345',
        status: null,
        updatedAt: '2017-10-31 08:54:06'
      })
    })
    it('should set hasListLoaded to true', async () => {
      const store = createStoreForTesting()
      const userId = '25'
      store.dispatch(UserActions.setCurrentUser(userId))
      store.addResponse(projectListRecord)
      expect(selectors.hasListLoaded(store.getState())).toBe(false)
      await store.dispatch(actions.fetchList())
      expect(selectors.hasListLoaded(store.getState())).toBe(true)
    })
  })
  describe('fetchInfo', () => {
    it('should do the correct api call', async () => {
      const store = createStoreForTesting()
      const userId = '25'
      const projectId = '593'
      store.dispatch(UserActions.setCurrentUser(userId))
      store.addResponse(projectInfoRecord)
      await store.dispatch(actions.fetchInfo({ projectId }))
      expect(fetch).toHaveBeenCalledWith(
        `${API_URL}/get/project/${projectId}/${userId}`,
        {
          headers: {
            Accept: 'application/json',
            'Content-Type': 'application/json'
          },
          method: 'GET'
        }
      )
    })
    it('should load a project and related resources', async () => {
      const store = createStoreForTesting()
      const userId = '25'
      const projectId = '768'
      const projectSelector = selectors.getProject(() => projectId)
      store.dispatch(UserActions.setCurrentUser(userId))
      store.addResponse(projectInfoRecord)
      await store.dispatch(actions.fetchInfo({ projectId }))
      const project = projectSelector(store.getState())
      const contacts = ContactSelectors.contacts(store.getState())
      expect(project.toJS()).toEqual({
        id: '768',
        name: 'Stubbs water claim',
        isSub: 0,
        long: -80.380851,
        lat: 25.670614,
        link: '',
        location: '',
        phoneNumber: '22334455',
        status: null,
        updatedAt: '2017-10-31 08:54:06',
        owner: '768',
        contacts: ['599', '600']
      })
      expect(contacts.getIn([project.getIn(['contacts', 0])]).toJS()).toEqual({
        id: '599',
        projectId: '593',
        name: 'Pikachu',
        phone: '223451',
        email: '',
        userId: '768',
        deleted: true,
        createdAt: '2017-10-31 08:54:06'
      })
    })
    it('should set hasLoaded to true', async () => {
      const store = createStoreForTesting()
      const userId = '25'
      const projectId = '768'
      const hasProjectLoaded = selectors.hasProjectLoaded(() => projectId)
      store.dispatch(UserActions.setCurrentUser(userId))
      store.addResponse(projectInfoRecord)
      expect(hasProjectLoaded(store.getState())).toBe(false)
      await store.dispatch(actions.fetchInfo({ projectId }))
      expect(hasProjectLoaded(store.getState())).toBe(true)
    })
    it('should leave the list of projects untouched', async () => {
      const store = createStoreForTesting()
      const userId = '25'
      const projectId = '768'
      store.dispatch(UserActions.setCurrentUser(userId))
      store.addResponse(projectListRecord)
      await store.dispatch(actions.fetchList())
      expect(selectors.projectsList(store.getState()).toJS()).toEqual([
        '593',
        '592',
        '591',
        '590'
      ])
      store.addResponse(projectInfoRecord)
      await store.dispatch(actions.fetchInfo({ projectId }))
      expect(selectors.projectsList(store.getState()).toJS()).toEqual([
        '593',
        '592',
        '591',
        '590'
      ])
    })
  })
})
