import createStoreForTesting from 'test/createstore'
import { API_URL } from '../../../config'
import * as actions from '../actions'
import * as selectors from '../selectors'
import { BidSelectors } from 'resources/bid'
import { SubSelectors } from 'resources/sub'
import projectTasksRecord from 'test/records/projecttasks.json'
import projectTaskSubRecord from 'test/records/projecttasksub.json'

describe('Task Resource', () => {
  describe('fetchForProject', () => {
    it('should do the correct api call', async () => {
      const store = createStoreForTesting()
      const projectId = '25'
      store.addResponse(projectTasksRecord)
      store.addResponse(projectTaskSubRecord)
      try {
        await store.dispatch(actions.fetchForProject({ projectId }))
      } catch (err) {
        console.log(err)
      }
      expect(fetch).toHaveBeenCalledWith(
        `${API_URL}/tasks/project/${projectId}`,
        {
          headers: {
            Accept: 'application/json',
            'Content-Type': 'application/json'
          },
          method: 'GET'
        }
      )
    })
    it('should merge state as expected', async () => {
      const store = createStoreForTesting()
      const projectId = '25'
      store.addResponse(projectTasksRecord)
      store.addResponse(projectTaskSubRecord)
      await store.dispatch(actions.fetchForProject({ projectId }))
      expect(selectors.tasks(store.getState()).toJS()).toEqual({
        '48': {
          id: '48',
          customTaskName: null,
          name: 'Unassigned',
          totalCost: 0,
          projectTask: '877',
          bids: ['691'],
          subs: ['828'],
          categories: [],
          url: '',
          taskSub: [
            {
              bid: null,
              bidStatus: 'No bids sent',
              cell: '3479850581',
              email: '+13479850581@dbmkllc.com',
              id: '771',
              name: 'Daniele Zannotti',
              reviews: 0,
              token: null
            }
          ]
        }
      })
      expect(
        selectors
          .getProject(() => projectId)(store.getState())
          .toJS()
      ).toEqual(['48'])
    })
    it('should merge related entities', async () => {
      const store = createStoreForTesting()
      const projectId = '25'
      const taskId = projectTasksRecord.data.tasks[0].id.toString()
      store.addResponse(projectTasksRecord)
      store.addResponse(projectTaskSubRecord)
      await store.dispatch(actions.fetchForProject({ projectId }))
      const task = selectors.tasks(store.getState()).get(taskId)
      const bidId = task.getIn(['bids', 0])
      const subId = task.getIn(['subs', 0])
      const bid = BidSelectors.bids(store.getState()).get(bidId)
      const sub = SubSelectors.subs(store.getState()).get(subId)
      expect(bid.toJS()).toMatchSnapshot()
      expect(sub.toJS()).toMatchSnapshot()
    })
    it('should not overwrite previous _resource', async () => {
      const store = createStoreForTesting()
      const projectId = '22'
      const otherProjectId = '44'
      const modifiedRecord = {
        data: { tasks: [...projectTasksRecord.data.tasks] }
      }
      modifiedRecord.data.tasks[0] = {
        ...projectTasksRecord.data.tasks[0],
        id: '28'
      }
      store.addResponse(modifiedRecord)
      store.addResponse(projectTaskSubRecord)
      await store.dispatch(actions.fetchForProject({ projectId }))
      store.addResponse(projectTasksRecord)
      store.addResponse(projectTaskSubRecord)
      await store.dispatch(
        actions.fetchForProject({ projectId: otherProjectId })
      )
      expect(
        selectors
          .getProject(() => projectId)(store.getState())
          .toJS()
      ).toEqual(['28'])
      expect(
        selectors
          .getProject(() => otherProjectId)(store.getState())
          .toJS()
      ).toEqual(['48'])
    })
  })
  describe('fetchList', () => {
    it('should merge state as expected', async () => {
      const store = createStoreForTesting()
      store.addResponse(projectTasksRecord)
      store.addResponse(projectTaskSubRecord)
      await store.dispatch(actions.fetchList())
      expect(selectors.tasks(store.getState()).toJS()).toEqual({
        '48': {
          id: '48',
          customTaskName: null,
          name: 'Unassigned',
          totalCost: 0,
          projectTask: '877',
          bids: ['691'],
          subs: ['828'],
          categories: [],
          taskSub: [],
          url: ''
        }
      })
      expect(selectors.taskList(store.getState()).toJS()).toEqual(['48'])
    })
  })
})
