// @flow
import { Alert } from 'react-native'
import { fromJS, List, Record } from 'immutable'
import { createSelector } from 'reselect'
import createReducer from 'utils/createreducer'
import createConstants from 'utils/createconstants'
import createViewSelector from 'utils/createviewselector'
import { ContactActions, ContactSelectors } from 'resources/contact'
import { ConversationActions } from 'resources/conversation'
import { TaskActions } from 'resources/task'
import { RouteActions } from 'routes'
import SCREENS from 'routes/screens'
import phoneMatch from 'utils/phonematch'
import sanitizePhoneNumber from 'utils/sanitizephone'
import sleep from 'utils/sleep'

export const CONSTANTS = createConstants('SCREEN_ADD_TO_CONVERSATION', {
  RESET: null,
  SET_SEARCH: null,
  TOGGLE_SELECTION: null,
  SET_PROJECT: null,
  SET_CONVERSATION: null,
  SET_REFRESH_INBOX: null
})

export const actions = {
  initialize ({ projectId, conversationId, refreshInbox }) {
    return dispatch => {
      dispatch(actions.reset())
      dispatch(actions.setProject(projectId))
      dispatch(actions.setConversation(conversationId))
      dispatch(actions.setRefreshInbox(refreshInbox))
      dispatch(
        RouteActions.setParams({
          add: () => dispatch(actions.addSelectionToConversation())
        })
      )
      dispatch(ContactActions.fetchRecentList())
      dispatch(actions.setSearch(''))
    }
  },

  reset () {
    return {
      type: CONSTANTS.RESET
    }
  },

  setProject (projectId) {
    return {
      type: CONSTANTS.SET_PROJECT,
      payload: projectId
    }
  },
  setRefreshInbox (refreshInbox) {
    return {
      type: CONSTANTS.SET_REFRESH_INBOX,
      payload: refreshInbox
    }
  },
  setConversation (conversationId) {
    return {
      type: CONSTANTS.SET_CONVERSATION,
      payload: conversationId
    }
  },
  setSearch (needle) {
    return {
      type: CONSTANTS.SET_SEARCH,
      payload: needle
    }
  },

  toggleContact (contactId) {
    return {
      type: CONSTANTS.TOGGLE_SELECTION,
      payload: contactId
    }
  },

  addContact ({ name, cell, email }) {
    return async (dispatch, getState) => {
      const projectId = selectors.projectId(getState())
      await dispatch(
        ContactActions.addContact({
          name,
          phone: sanitizePhoneNumber(cell),
          email: '',
          projectId
        })
      )
      await dispatch(ContactActions.fetchRecentList())
      dispatch(
        actions.addToConversation({ name, cell: sanitizePhoneNumber(cell) })
      )
    }
  },

  toAddContact () {
    return (dispatch, getState) => {
      dispatch(
        RouteActions.navigate(SCREENS.ADD_MANUAL_CONTACT, {
          onSave: actions.addContact,
          projectId: '0'
        })
      )
    }
  },

  toAddressBookContact () {
    return (dispatch, getState) => {
      dispatch(
        RouteActions.navigate(SCREENS.ADD_ADDRESSBOOK_CONTACT, {
          onSave: actions.addContact,
          projectId: '0'
        })
      )
    }
  },
  addSelectionToConversation () {
    return async (dispatch, getState) => {
      const selection = selectors.selection(getState())
      if (!selection.size) {
        Alert.alert('Nailit!', 'Select a recent contact or add a new one')
        return
      }
      const contact = ContactSelectors.contacts(getState()).get(
        selection.get(0)
      )
      dispatch(
        actions.addToConversation({
          name: contact.get('name'),
          cell: contact.get('phone')
        })
      )
    }
  },
  addToConversation ({ name, cell }) {
    return async (dispatch, getState) => {
      const saneCell = sanitizePhoneNumber(cell)
      const projectId = selectors.projectId(getState())
      const conversationId = selectors.conversationId(getState())
      const refreshInbox = selectors.refreshInbox(getState())
      const contactDetails = await dispatch(
        TaskActions.addPeopleToTrade({
          taskId: 0,
          subs: [
            {
              name,
              cell: saneCell
            }
          ],
          projectId
        })
      )
      const subContact = contactDetails.data.subs.find(sub =>
        phoneMatch(sub.cell, saneCell)
      )
      const newConversation = await dispatch(
        ConversationActions.addToConversation({
          conversationId,
          conversationUserId: subContact.id
        })
      )
      const newConversationId =
        newConversation.data.conversationId ||
        newConversation.data.newConversationId
      await dispatch(refreshInbox())
      dispatch(RouteActions.back())
      await sleep(200)
      dispatch(RouteActions.back())
      await sleep(200)
      dispatch(
        RouteActions.navigate(SCREENS.CONVERSATION, {
          roomId: newConversation.data.roomId,
          conversationId: newConversationId
        })
      )
    }
  }
}

export const AddToConversationState = Record({
  search: '',
  projectId: null,
  conversationId: null,
  refreshInbox: null,
  selection: List()
})

export const reducer = createReducer(new AddToConversationState(), {
  [CONSTANTS.RESET]: state => new AddToConversationState(),
  [CONSTANTS.SET_SEARCH]: (state, { payload }) => state.set('search', payload),
  [CONSTANTS.SET_PROJECT]: (state, { payload }) =>
    state.set('projectId', payload),
  [CONSTANTS.SET_CONVERSATION]: (state, { payload }) =>
    state.set('conversationId', payload),
  [CONSTANTS.SET_REFRESH_INBOX]: (state, { payload }) =>
    state.set('refreshInbox', payload),

  [CONSTANTS.TOGGLE_SELECTION]: (state, { payload }) => {
    const selection = state.get('selection')
    if (selection.includes(payload)) {
      const newSelection = selection.filter(id => id !== payload)
      return state.set('selection', newSelection)
    }
    return state.set('selection', fromJS([payload]))
  }
})

export const viewState = state => state.screens.addtoconversation
export const viewSelector = createViewSelector(viewState)
export const search = viewSelector('search')
export const selection = viewSelector('selection')
export const projectId = viewSelector('projectId')
export const conversationId = viewSelector('conversationId')
export const refreshInbox = viewSelector('refreshInbox')

export const contactsList = createSelector(
  [ContactSelectors.recentContactsList, search, ContactSelectors.contacts],
  (recentContacts, search, contacts) =>
    recentContacts.filter(
      contactId =>
        contacts
          .getIn([contactId, 'name'])
          .toLowerCase()
          .indexOf(search.toLowerCase()) !== -1
    )
)

export const selectors = {
  search,
  selection,
  projectId,
  refreshInbox,
  conversationId,
  contacts: ContactSelectors.contacts,
  contactsList,
  hasLoaded: ContactSelectors.hasRecentContactsListLoaded
}
