// @flow
import PropTypes from 'prop-types'
import React from 'react'
import { Linking, View } from 'react-native'
import StyleSheet from 'react-native-extended-stylesheet'
import ParsedText from 'react-native-parsed-text'
import Communications from 'vendor/react-native-communications'

const WWW_URL_PATTERN = /^www\./i

const textStyle = {
  fontSize: 15,
  lineHeight: 20,
  marginTop: 8,
  marginBottom: 5,
  marginLeft: 10,
  marginRight: 10
}

const styles = {
  left: StyleSheet.create({
    container: {},
    text: {
      color: 'black',
      ...textStyle
    },
    link: {
      color: 'black',
      textDecorationLine: 'underline'
    }
  }),
  right: StyleSheet.create({
    container: {},
    text: {
      color: 'white',
      ...textStyle
    },
    link: {
      color: 'white',
      textDecorationLine: 'underline'
    }
  })
}

export default class MessageText extends React.Component {
  static propTypes = {
    color: PropTypes.string,
    position: PropTypes.oneOf(['left', 'right']),
    message: PropTypes.string,
    hasImage: PropTypes.bool.isRequired
  }

  static defaultProps = {
    position: 'left'
  }

  constructor (props) {
    super(props)
    this.onUrlPress = this.onUrlPress.bind(this)
    this.onPhonePress = this.onPhonePress.bind(this)
    this.onEmailPress = this.onEmailPress.bind(this)
  }

  onUrlPress (url) {
    // When someone sends a message that includes a website address beginning with "www." (omitting the scheme),
    // react-native-parsed-text recognizes it as a valid url, but Linking fails to open due to the missing scheme.
    if (WWW_URL_PATTERN.test(url)) {
      this.onUrlPress(`http://${url}`)
    } else {
      Linking.canOpenURL(url).then(supported => {
        if (!supported) {
          console.error('No handler for URL:', url)
        } else {
          Linking.openURL(url)
        }
      })
    }
  }

  onPhonePress (phone) {
    Communications.phonecall(phone, true)
  }

  onEmailPress (email) {
    Communications.email([email], null, null, null, null)
  }

  render () {
    const { color, message, position, hasImage } = this.props
    const linkStyle = styles[position].link
    if (!message || !message.trim() || hasImage) {
      return null
    }
    return (
      <View style={styles[position].container}>
        <ParsedText
          style={[styles[position].text, { color }]}
          parse={[
            { type: 'url', style: linkStyle, onPress: this.onUrlPress },
            { type: 'phone', style: linkStyle, onPress: this.onPhonePress },
            { type: 'email', style: linkStyle, onPress: this.onEmailPress }
          ]}
        >
          {message}
        </ParsedText>
      </View>
    )
  }
}
