// @flow
import { List, Record } from 'immutable'
import { createSelector } from 'reselect'
import { createAction } from 'redux-actions'
import { last } from 'lodash'
import humps from 'humps'
import { Alert, PermissionsAndroid, Platform } from 'react-native'
import ActionSheet from '@yfuks/react-native-action-sheet'
import createConstants from 'utils/createconstants'
import createReducer from 'utils/createreducer'
import createViewSelector from 'utils/createviewselector'
import Communications from 'vendor/react-native-communications'
import {
  ConversationActions,
  ConversationSelectors,
  MESSAGE_TYPE
} from 'resources/conversation'
import { UserSelectors } from 'resources/user'
import { ProjectSelectors } from 'resources/project'
import { TaskActions, TaskSelectors } from 'resources/task'
import { InboxActions } from 'screens/Inbox'
import { InboxActions as InboxResourceAction } from 'resources/inbox'
import { RouteActions } from 'routes'
import SCREENS from 'routes/screens'
import SocketIOClient from 'socket.io-client'
import { AudioRecorder, AudioUtils } from 'vendor/react-native-audio'
import Sound from 'react-native-sound'
import { SOCKET_URL } from '../../config'
import fileChooser from 'utils/filechooser'

export const CONSTANTS = createConstants('VIEW_CONVERSATION', {
  SET_TEXT: null,
  SET_CONVERSATION_ID: null,
  SET_ROOM_ID: null,
  SET_SOCKET: null,
  SET_RECORD_MODAL: null,
  SET_RECORDING: null,
  SET_PLAYING: null,
  SET_CURRENT_TIME: null,
  SET_VOLUME_WIDTH: null,
  SET_FILE_NAME: null,
  SET_SOUND: null,
  SET_AUDIO_CLIP: null,
  SET_REFRESH_INBOX: null,
  SET_AUDIO_LOADING: null
})

export const actions = {
  initialize ({ conversationId, roomId, refreshInbox }) {
    return dispatch => {
      dispatch(actions.loadConversation({ conversationId, roomId }))
      dispatch(actions.setText(''))
      dispatch(actions.setRefreshInbox(refreshInbox))
      dispatch(InboxResourceAction.markRead({ conversationId }))
      console.log('going', conversationId)
      if (Platform.OS !== 'android') {
        return
      }

      const rationale = {
        title: 'Microphone Permission',
        message:
          'Nailit needs access to your microphone so you can record audio.'
      }
      return PermissionsAndroid.request(
        PermissionsAndroid.PERMISSIONS.RECORD_AUDIO,
        rationale
      )
    }
  },
  loadConversation ({ conversationId, roomId }) {
    return async (dispatch, getState) => {
      dispatch(actions.setConversationId(conversationId))
      dispatch(actions.setRoomId(roomId))
      await dispatch(ConversationActions.loadConversation({ conversationId }))
      const state = getState()
      const users = UserSelectors.users(state)
      const projectId = selectors.projectId(getState())
      const currentUserId = UserSelectors.currentUserId(state)
      const currentUser = users.get(currentUserId)
      const conversation = ConversationSelectors.getConversation(
        selectors.conversationId
      )(state)
      const partecipantsId = conversation
        .get('partecipants')
        .filter(userId => userId !== currentUserId)
      const partecipantsName = partecipantsId.map(userId =>
        users.getIn([userId, 'name'])
      )
      dispatch(
        RouteActions.setTitle(
          partecipantsName.join(', ') || currentUser.get('name')
        )
      )
      await dispatch(TaskActions.fetchForProject({ projectId }))
      dispatch(actions.websocketSubscribe())
    }
  },
  setText: createAction(CONSTANTS.SET_TEXT),
  setConversationId: createAction(CONSTANTS.SET_CONVERSATION_ID),
  setRoomId: createAction(CONSTANTS.SET_ROOM_ID),

  sendMessage () {
    return (dispatch, getState) => {
      const text = selectors.text(getState()).trim()
      const conversationId = selectors.conversationId(getState())
      if (!text) {
        return
      }
      dispatch(
        ConversationActions.sendMessage({
          conversationId,
          text
        })
      )
      dispatch(actions.setText(''))
    }
  },
  setRefreshInbox (refreshInbox) {
    return {
      type: CONSTANTS.SET_REFRESH_INBOX,
      payload: refreshInbox
    }
  },
  toProjectInbox () {
    return (dispatch, getState) => {
      const projectId = selectors.projectId(getState())
      dispatch(InboxActions.setProject(projectId))
      dispatch(RouteActions.back())
    }
  },

  toProjectDetails () {
    return (dispatch, getState) => {
      const projectId = selectors.projectId(getState())
      dispatch(RouteActions.navigate(SCREENS.PROJECT_INFO, { projectId }))
    }
  },

  showMessageOptions () {
    return (dispatch, getState) => {
      const state = getState()
      const conversation = selectors.conversation(state)
      if (!conversation) {
        return
      }
      const currentUserId = UserSelectors.currentUserId(state)
      const isGroupConversation =
        conversation
          .get('partecipants')
          .filter(userId => userId !== currentUserId).size > 1
      const options = [
        'Call',
        'Send Photo from Camera',
        'Send Photo from Library',
        'Send Photo from Dropbox',
        'Add to Conversation',
        !isGroupConversation && 'Assign Trade',
        'Cancel'
      ].filter(value => value)
      ActionSheet.showActionSheetWithOptions(
        {
          options,
          cancelButtonIndex: options.length - 1
        },
        buttonIndex => {
          if (buttonIndex === options.length - 1) {
            return
          }
          switch (buttonIndex) {
            case 0:
              dispatch(actions.call())
              break
            case 1:
              dispatch(actions.sendPhoto({ camera: true }))
              break
            case 2:
              dispatch(actions.sendPhoto({ library: true }))
              break
            case 3:
              dispatch(actions.sendPhoto({ images: true }))
              break
            case 4:
              dispatch(actions.showAddToConversation())
              break
            case 5:
              dispatch(actions.showAssignTrade())
              break
          }
        }
      )
    }
  },

  // options
  call () {
    return async (dispatch, getState) => {
      const state = getState()
      const users = UserSelectors.users(state)
      const currentUserId = UserSelectors.currentUserId(state)
      const conversationId = selectors.conversationId(state)
      const conversation = ConversationSelectors.getConversation(
        selectors.conversationId
      )(state)
      if (!conversation) {
        Alert.alert(
          'Nailit',
          'Conversation is still loading, try again in a few seconds'
        )
        return
      }
      const partecipantsId = conversation
        .get('partecipants')
        .filter(userId => userId !== currentUserId)
      if (partecipantsId.size === 1) {
        const user = users.get(partecipantsId.get(0))
        try {
          await Communications.phonecall(user.get('cell'), true)
          dispatch(
            ConversationActions.sendMessage({
              conversationId,
              type: MESSAGE_TYPE.PHONE_CALL
            })
          )
        } catch (err) {
          console.log('conversation error', err)
        }
        return
      }
      ActionSheet.showActionSheetWithOptions(
        {
          options: partecipantsId
            .map(userId => users.getIn([userId, 'name']))
            .push('Cancel')
            .toJS()
        },
        buttonIndex => {
          if (buttonIndex === partecipantsId.size) {
            return
          }
          const user = users.get(partecipantsId.get(buttonIndex))
          Communications.phonecall(user.get('cell'), true)
          dispatch(
            ConversationActions.sendMessage({
              conversationId,
              type: MESSAGE_TYPE.PHONE_CALL
            })
          )
        }
      )
    }
  },

  sendPhoto (chooserParams) {
    return async (dispatch, getState) => {
      try {
        const images = await fileChooser(chooserParams)
        images.forEach(image => {
          const source = image
          const filename = last(image.split('/'))
          // TODO: show loader
          const projectId = selectors.projectId(getState())
          const conversationId = selectors.conversationId(getState())
          const project = ProjectSelectors.getProject(selectors.projectId)(
            getState()
          )
          const projectLat = project.get('lat')
          const projectLong = project.get('long')
          const projectName = project.get('name')
          const projectLocation = project.get('location')
          dispatch(
            ConversationActions.uploadMessageImage({
              conversationId,
              projectId,
              projectLat,
              projectLong,
              projectLocation,
              projectName,
              filename,
              filepath: source
            })
          )
        })
      } catch (error) {
        console.log(error)
      }
    }
  },

  showAssignTrade () {
    return (dispatch, getState) => {
      const state = getState()
      const conversation = selectors.conversation(state)
      if (!conversation) {
        return
      }
      const currentUserId = UserSelectors.currentUserId(state)
      const isGroupConversation =
        conversation
          .get('partecipants')
          .filter(userId => userId !== currentUserId).size > 1
      if (isGroupConversation) {
        return
      }
      const partecipantUserId = conversation
        .get('partecipants')
        .filter(userId => userId !== currentUserId)
        .get(0)

      const tasks = TaskSelectors.tasks(state)
      const taskList = TaskSelectors.getProject(selectors.projectId)(state)
      const tradeSelected = taskList.reduce((list, taskId) => {
        const task = tasks.get(taskId)
        const subs = task.get('taskSub', new List())
        const assignedTask = subs.find(
          sub => sub.get('id') === partecipantUserId
        )
        if (assignedTask) {
          list.push(taskId)
        }
        return list
      }, [])
      dispatch(
        RouteActions.navigate(SCREENS.SELECT_TRADE, {
          save: actions.saveTrades,
          singleSelection: false,
          selected: tradeSelected
        })
      )
    }
  },

  saveTrades (trades) {
    return async (dispatch, getState) => {
      let state = getState()
      const currentUserId = UserSelectors.currentUserId(state)
      const users = UserSelectors.users(state)
      const conversation = selectors.conversation(state)
      const projectId = conversation.get('project')
      await dispatch(TaskActions.fetchForProject({ projectId }))
      state = getState()
      const tasks = TaskSelectors.tasks(state)
      const taskList = TaskSelectors.getProject(selectors.projectId)(state)
      // this is single user conversation only, so subs is ultimately the other user
      const subs = conversation
        .get('partecipants')
        .filter(userId => userId !== currentUserId)
        .map(subId => ({
          name: users.getIn([subId, 'name']),
          cell: users.getIn([subId, 'cell'])
        }))
        .toJS()
      const partecipantUserId = conversation
        .get('partecipants')
        .filter(userId => userId !== currentUserId)
        .get(0)
      const oldTradeSelected = taskList.reduce((list, taskId) => {
        const task = tasks.get(taskId)
        const subs = task.get('taskSub', new List())
        const assignedTask = subs.find(
          sub => sub.get('id') === partecipantUserId
        )
        if (assignedTask) {
          list.push(taskId)
        }
        return list
      }, [])
      // diff old trade and new trades and apply to api
      const addTrades = trades
        ? trades
          .filter(x => oldTradeSelected.indexOf(x) < 0)
          .toJS()
          .filter(v => v)
        : []
      const removedTrades = oldTradeSelected
        .filter(trade => trades.indexOf(trade) < 0)
        .map(trade => tasks.getIn([trade, 'projectTask']))
        .filter(trade => trade)
      if (removedTrades.length) {
        await Promise.all(
          removedTrades.map(projectTask =>
            dispatch(
              TaskActions.removePeopleFromTrade({
                taskId: projectTask,
                subs: [{ userId: partecipantUserId }],
                projectId
              })
            )
          )
        )
      }
      if (addTrades.length) {
        await Promise.all(
          addTrades.map(trade =>
            dispatch(
              TaskActions.addPeopleToTrade({
                taskId: trade,
                subs,
                projectId
              })
            )
          )
        )
      }
    }
  },

  showAddToConversation () {
    return (dispatch, getState) => {
      const state = getState()
      const conversation = selectors.conversation(state)
      if (!conversation) {
        return
      }
      const projectId = conversation.get('project')
      const conversationId = selectors.conversationId(getState())
      const refreshInbox = selectors.refreshInbox(getState())
      dispatch(
        RouteActions.navigate(SCREENS.ADD_TO_CONVERSATION, {
          conversationId,
          projectId,
          refreshInbox
        })
      )
    }
  },

  // websocket management
  websocketSubscribe () {
    return (dispatch, getState) => {
      const roomId = selectors.roomId(getState())
      const socket = SocketIOClient(SOCKET_URL, {
        query: 'room=' + roomId,
        jsonp: false,
        reconnect: true,
        multiplex: false,
        transports: ['websocket']
      })
      console.log('socket connecting', SOCKET_URL, roomId)
      // socket callbacks
      socket.on('connect', () =>
        console.log(`socket connect to conversation room: ${roomId}`)
      )
      socket.on('disconnect', () => console.log('socket disconnect'))
      socket.on('reconnect', () => console.log('socket reconnect'))
      socket.on(roomId, data => dispatch(actions.websocketReceive(data)))

      // store the socket in state
      dispatch({
        type: CONSTANTS.SET_SOCKET,
        payload: socket
      })
    }
  },
  websocketUnsubscribe () {
    return (dispatch, getState) => {
      setTimeout(() => {
        const socket = selectors.socket(getState())
        if (socket) {
          socket.disconnect()
        }
      }, 150)
    }
  },

  websocketReconnect () {
    return (dispatch, getState) => {
      dispatch(actions.websocketSubscribe())
      dispatch(actions.websocketSubscribe())
    }
  },

  websocketReceive (dataString) {
    return (dispatch, getState) => {
      const message = humps.camelizeKeys(JSON.parse(dataString))
      const conversationId = selectors.conversationId(getState())
      dispatch(
        ConversationActions.receivedSocketMessage({
          ...message,
          conversationId,
          text: message.message
        })
      )
    }
  },

  // audio management
  openRecordAudio () {
    return dispatch => {
      dispatch(actions.setAudioClip(false))
      dispatch({ type: CONSTANTS.SET_PLAYING, payload: false })
      dispatch({ type: CONSTANTS.SET_RECORDING, payload: false })
      dispatch({
        type: CONSTANTS.SET_RECORD_MODAL,
        payload: true
      })
    }
  },

  sendRecord () {
    return async (dispatch, getState) => {
      await dispatch(actions.stopClip())
      dispatch({ type: CONSTANTS.SET_PLAYING, payload: false })
      dispatch({ type: CONSTANTS.SET_RECORDING, payload: false })
      const hasAudioClip = selectors.hasAudioClip(getState())
      if (!hasAudioClip) {
        Alert.alert('Nailit!', 'No clip recorded')
        return
      }
      const conversationId = selectors.conversationId(getState())
      const filename =
        Platform.OS === 'ios'
          ? selectors.filename(getState())
          : `${AudioUtils.DocumentDirectoryPath}/test.aac`
      const projectId = selectors.projectId(getState())
      dispatch(actions.setAudioLoading(true))
      try {
        await dispatch(
          ConversationActions.uploadMessageAudio({
            conversationId,
            duration: '0',
            projectId,
            filename: 'audiomessage.caf',
            filepath: filename
          })
        )
      } catch (error) {
        Alert.alert('Nailit!', 'Could not upload the audio message')
        console.log('sendRecord error', error)
      } finally {
        dispatch(actions.setAudioLoading(false))
      }
      dispatch(actions.setAudioClip(false))
      const sound = selectors.sound(getState())
      if (sound) {
        sound.release()
      }
      dispatch(actions.setSound(null))
      dispatch({
        type: CONSTANTS.SET_RECORD_MODAL,
        payload: false
      })
    }
  },

  cancelRecord () {
    return (dispatch, getState) => {
      dispatch(actions.setAudioClip(false))
      let sound = selectors.sound(getState())
      if (sound) {
        sound.release()
      }
      dispatch({ type: CONSTANTS.SET_PLAYING, payload: false })
      dispatch({ type: CONSTANTS.SET_RECORDING, payload: false })
      dispatch({
        type: CONSTANTS.SET_RECORD_MODAL,
        payload: false
      })
    }
  },

  recordClip () {
    return async dispatch => {
      dispatch({ type: CONSTANTS.SET_PLAYING, payload: false })
      dispatch({ type: CONSTANTS.SET_RECORDING, payload: true })
      const recordingFile = AudioUtils.DocumentDirectoryPath + '/test.aac'
      dispatch(actions.setFilename(recordingFile))
      dispatch(actions.setSound(null))
      AudioRecorder.onProgress = ({ currentTime, currentMetering }) => {
        let meterValue = Math.abs(currentMetering)
        if (meterValue < 0) {
          meterValue = 0
        } else if (meterValue > 100) {
          meterValue = 100
        }
        dispatch(actions.setVolumeWidth(meterValue))
        dispatch(actions.setCurrentTime(currentTime))
      }
      AudioRecorder.onFinished = data => {
        if (Platform.OS === 'ios') {
          dispatch(actions.setFilename(data.audioFileURL))
        } else {
          dispatch(actions.setFilename(recordingFile))
        }
      }

      AudioRecorder.prepareRecordingAtPath(recordingFile, {
        SampleRate: 22050,
        Channels: 1,
        AudioQuality: 'Low',
        AudioEncoding: 'aac',
        AudioEncodingBitRate: 32000,
        MeteringEnabled: true
      })
      try {
        await AudioRecorder.startRecording()
      } catch (error) {
        console.log('audiorecorder', error)
      }
    }
  },

  setFilename (filename) {
    return {
      type: CONSTANTS.SET_FILE_NAME,
      payload: filename
    }
  },

  setVolumeWidth (width) {
    return {
      type: CONSTANTS.SET_VOLUME_WIDTH,
      payload: width
    }
  },

  setCurrentTime (time) {
    return {
      type: CONSTANTS.SET_CURRENT_TIME,
      payload: time
    }
  },

  setAudioLoading (loading) {
    return {
      type: CONSTANTS.SET_AUDIO_LOADING,
      payload: loading
    }
  },

  playClip () {
    return (dispatch, getState) => {
      let sound = selectors.sound(getState())
      if (sound) {
        sound.release()
      }
      dispatch({ type: CONSTANTS.SET_PLAYING, payload: true })
      dispatch({ type: CONSTANTS.SET_RECORDING, payload: false })
      const filename =
        Platform.OS === 'ios'
          ? selectors.filename(getState())
          : `${AudioUtils.DocumentDirectoryPath}/test.aac`
      // These timeouts are a hacky workaround for some issues with react-native-sound.
      // See https://github.com/zmxv/react-native-sound/issues/89.
      setTimeout(() => {
        sound = new Sound(filename, '', error => {
          if (error) {
            console.log('failed to load the sound', error)
          }
        })
        sound.setVolume(0.5)
        sound.setCategory('Playback')
        dispatch(actions.setSound(sound))

        setTimeout(() => {
          sound.play(success => {
            if (success) {
              dispatch({ type: CONSTANTS.SET_PLAYING, payload: false })
            }
          })
        }, 100)
      }, 100)
    }
  },

  pauseClip () {
    return (dispatch, getState) => {
      dispatch({ type: CONSTANTS.SET_PLAYING, payload: false })
      dispatch({ type: CONSTANTS.SET_RECORDING, payload: false })
      const sound = selectors.sound(getState())
      if (sound) {
        sound.pause()
      }
    }
  },

  stopClip () {
    return async (dispatch, getState) => {
      const isRecording = selectors.isRecording(getState())
      const isPlaying = selectors.isPlaying(getState())
      const sound = selectors.sound(getState())
      dispatch({ type: CONSTANTS.SET_PLAYING, payload: false })
      dispatch({ type: CONSTANTS.SET_RECORDING, payload: false })
      if (isRecording) {
        try {
          await AudioRecorder.stopRecording()
          dispatch(actions.setAudioClip(true))
        } catch (error) {
          console.log('stop error', error)
        }
      }
      if (isPlaying && sound) {
        sound.stop()
      }
    }
  },

  setSound (sound) {
    return {
      type: CONSTANTS.SET_SOUND,
      payload: sound
    }
  },

  setAudioClip (value) {
    return {
      type: CONSTANTS.SET_AUDIO_CLIP,
      payload: value
    }
  }
}

const ConversationState = Record({
  text: '',
  conversationId: null,
  roomId: null,
  socket: null,
  isRecording: false,
  showRecordModal: false,
  isPlaying: null,
  volumeWidth: 0,
  currentTime: 0,
  audioClip: null,
  filename: null,
  sound: null,
  refreshInbox: null,
  isAudioLoading: false
})

export const reducer = createReducer(new ConversationState(), {
  [CONSTANTS.SET_TEXT] (state, { payload }) {
    return state.set('text', payload)
  },
  [CONSTANTS.SET_CONVERSATION_ID] (state, { payload }) {
    return state.set('conversationId', payload)
  },
  [CONSTANTS.SET_SOCKET] (state, { payload }) {
    return state.set('socket', payload)
  },
  [CONSTANTS.SET_ROOM_ID] (state, { payload }) {
    return state.set('roomId', payload)
  },
  [CONSTANTS.SET_RECORD_MODAL] (state, { payload }) {
    return state.set('showRecordModal', payload)
  },
  [CONSTANTS.SET_PLAYING] (state, { payload }) {
    return state.set('isPlaying', payload)
  },
  [CONSTANTS.SET_RECORDING] (state, { payload }) {
    return state.set('isRecording', payload)
  },
  [CONSTANTS.SET_CURRENT_TIME] (state, { payload }) {
    return state.set('currentTime', payload)
  },
  [CONSTANTS.SET_VOLUME_WIDTH] (state, { payload }) {
    return state.set('volumeWidth', payload)
  },
  [CONSTANTS.SET_FILE_NAME] (state, { payload }) {
    return state.set('filename', payload)
  },
  [CONSTANTS.SET_SOUND] (state, { payload }) {
    return state.set('sound', payload)
  },
  [CONSTANTS.SET_AUDIO_CLIP] (state, { payload }) {
    return state.set('audioClip', payload)
  },
  [CONSTANTS.SET_AUDIO_LOADING] (state, { payload }) {
    return state.set('isAudioLoading', payload)
  },
  [CONSTANTS.SET_REFRESH_INBOX] (state, { payload }) {
    return state.set('refreshInbox', payload)
  }
})

export const viewState = state => state.screens.conversation
export const viewSelector = createViewSelector(viewState)

export const conversationId = viewSelector('conversationId')
export const roomId = viewSelector('roomId')
export const text = viewSelector('text', '')
export const socket = viewSelector('socket')
export const isRecording = viewSelector('isRecording')
export const showRecordModal = viewSelector('showRecordModal')
export const isPlaying = viewSelector('isPlaying')
export const audioClip = viewSelector('audioClip')
export const volumeWidth = viewSelector('volumeWidth')
export const currentTime = viewSelector('currentTime')
export const filename = viewSelector('filename')
export const sound = viewSelector('sound')
export const refreshInbox = viewSelector('refreshInbox')
export const isAudioLoading = viewSelector('isAudioLoading')
export const hasLoaded = ConversationSelectors.hasLoaded(conversationId)
export const isLoading = createSelector([hasLoaded], hasLoaded => !hasLoaded)
export const conversation = ConversationSelectors.getConversation(
  conversationId
)
export const projectId = createSelector(
  [conversation],
  conversation => conversation && conversation.get('project')
)
export const hasAudioClip = audioClip

export const renderableConversation = createSelector(
  [conversation],
  conversation => {
    if (!conversation) {
      return null
    }
    return conversation.set(
      'messages',
      conversation.get('messages', List()).filter(message => {
        const isText =
          (message.get('type') === MESSAGE_TYPE.APP ||
            message.get('type') === MESSAGE_TYPE.TEXT) &&
          message.get('message', '').trim()
        const isImage =
          (message.get('type') === MESSAGE_TYPE.IMAGE &&
            message.get('mediaThumbnail')) ||
          message.get('mediaUrl')
        const isCall = message.get('type') === MESSAGE_TYPE.PHONE_CALL
        const isAudio = message.get('type') === MESSAGE_TYPE.AUDIO
        return isText || isImage || isCall || isAudio
      })
    )
  }
)

export const isGroupConversation = createSelector(
  [conversation, UserSelectors.currentUserId],
  (conversation, currentUserId) =>
    conversation &&
    conversation
      .get('partecipants', List())
      .filter(userId => userId !== currentUserId).size > 1
)

export const selectors = {
  conversationId,
  projectId,
  text,
  socket,
  roomId,
  isLoading,
  isRecording,
  isGroupConversation,
  showRecordModal,
  audioClip,
  isPlaying,
  isAudioLoading,
  hasAudioClip,
  volumeWidth,
  currentTime,
  filename,
  refreshInbox,
  sound,
  users: UserSelectors.users,
  currentUserId: UserSelectors.currentUserId,
  conversation: renderableConversation
}
