// @flow
import { Alert } from 'react-native'
import { Record, List, fromJS } from 'immutable'
import createReducer from 'utils/createreducer'
import createConstants from 'utils/createconstants'
import createViewSelector from 'utils/createviewselector'
import { createSelector } from 'reselect'
import { ModalActions } from 'components/Modal'
import { ContactActions, ContactSelectors } from 'resources/contact'
import { InboxActions, InboxSelectors } from 'resources/inbox'
import { ConversationActions } from 'resources/conversation'
import { ProjectActions, ProjectSelectors } from 'resources/project'
import { TaskActions } from 'resources/task'
import isNumeric from 'utils/isnumeric'
import { RouteActions, SCREENS } from 'routes'

export const CONSTANTS = createConstants('SCREEN_CREATE_CONVERSATION', {
  RESET: null,
  SET_MESSAGE: null,
  TOGGLE_PROJECT: null,
  ADD_RECIPIENT: null,
  REMOVE_RECIPIENT: null,
  SET_LOADING: null
})

export const actions = {
  initialize () {
    return async dispatch => {
      try {
        dispatch(actions.reset())
        dispatch(ContactActions.fetchRecentList())
        dispatch(ProjectActions.fetchList())
      } catch (error) {
        console.log('create conversation initialize failed', error)
      }
    }
  },

  reset () {
    return {
      type: CONSTANTS.RESET
    }
  },

  addRecipient (recipient) {
    return {
      type: CONSTANTS.ADD_RECIPIENT,
      payload: recipient
    }
  },

  removeRecipient (recipientIndex) {
    return {
      type: CONSTANTS.REMOVE_RECIPIENT,
      payload: recipientIndex
    }
  },

  setMessage (message) {
    return {
      type: CONSTANTS.SET_MESSAGE,
      payload: message
    }
  },
  toggleProject (projectId) {
    return {
      type: CONSTANTS.TOGGLE_PROJECT,
      payload: projectId
    }
  },
  createConversation () {
    return async (dispatch, getState) => {
      const state = getState()
      const selectedProjects = selectors.selectedProjects(state)
      const recipients = selectors.recipients(state)
      const message = selectors.message(state)
      let target = []
      if (!selectedProjects.size) {
        Alert.alert('Nailit!', 'You must select a project')
        return
      }
      if (!recipients.size) {
        Alert.alert('Nailit!', 'You must add at least a recipient')
        return
      }
      if (!message.trim().length) {
        Alert.alert('Nailit!', 'You must add a message')
        return
      }
      for (let i = 0; i < recipients.size; i++) {
        try {
          let recipientValue = recipients.get(i)
          if (recipientValue.startsWith('+')) {
            recipientValue = recipientValue.substr(1)
          }
          if (recipientValue.startsWith('1')) {
            recipientValue = recipientValue.substr(1)
          }
          if (isNumeric(recipientValue)) {
            const name = await dispatch(
              ModalActions.confirm(`Name for ${recipients.get(i)}`, 0)
            )
            target.push({
              name,
              cell: recipients.get(i)
            })
          } else {
            // find from contacts
            const contacts = ContactSelectors.contacts(state)
            const recipient = contacts.find(
              contact => contact.get('name') === recipientValue
            )
            let cell
            if (!recipient || !recipient.get('phone')) {
              cell = await dispatch(
                ModalActions.confirm(
                  `Phone for ${recipients.get(i)}`,
                  10,
                  'Phone number is too short'
                )
              )
            } else {
              cell = recipient.get('phone')
            }
            target.push({
              name: recipientValue,
              cell
            })
          }
          // create conversation and back + redirect
          try {
            dispatch(actions.setLoading(true))
            // add to trade
            const tradesPeople = await dispatch(
              TaskActions.addPeopleToTrade({
                taskId: 0,
                subs: target,
                projectId: selectedProjects.get(0)
              })
            )
            const subsId = tradesPeople.data.subs
              .filter(sub => target.find(tar => tar.cell === sub.cell))
              .map(sub => sub.id)
            // add to conversation
            const conversationResponse = await dispatch(
              ConversationActions.createConversationWith({
                projectId: selectedProjects.get(0),
                partecipants: subsId,
                message
              })
            )
            await dispatch(
              ConversationActions.loadConversation({
                conversationId: conversationResponse.data.newConversationId
              })
            )
            await dispatch(
              InboxActions.fetchList({ projectId: selectedProjects.get(0) })
            )
            const inbox = InboxSelectors.getConversation(
              conversationResponse.data.newConversationId
            )(getState())
            dispatch(actions.setLoading(false))
            // back + redirect
            dispatch(RouteActions.back())
            dispatch(
              RouteActions.navigate(SCREENS.CONVERSATION, {
                conversationId: inbox.get('conversation'),
                roomId: inbox.get('room')
              })
            )
          } catch (error) {
            console.log(error)
            dispatch(actions.setLoading(false))
            Alert.alert(
              'Nailit!',
              'Something went wrong when creating the conversation'
            )
          }
        } catch (error) {
          console.log(error)
          return
        }
      }
    }
  },
  setLoading (value) {
    return {
      type: CONSTANTS.SET_LOADING,
      payload: value
    }
  }
}

export const CreateConversationState = Record({
  message: '',
  selectedProjects: List(),
  recipients: List(),
  isLoading: false
})

export const reducer = createReducer(new CreateConversationState(), {
  [CONSTANTS.RESET]: () => new CreateConversationState(),
  [CONSTANTS.SET_MESSAGE]: (state, { payload }) =>
    state.set('message', payload),
  [CONSTANTS.SET_LOADING]: (state, { payload }) =>
    state.set('isLoading', payload),

  [CONSTANTS.TOGGLE_PROJECT]: (state, { payload }) => {
    return state.set('selectedProjects', fromJS([payload]))
  },
  [CONSTANTS.ADD_RECIPIENT]: (state, { payload }) => {
    return state.set('recipients', state.get('recipients').push(payload))
  },
  [CONSTANTS.REMOVE_RECIPIENT]: (state, { payload }) => {
    return state.set('recipients', state.get('recipients').splice(payload, 1))
  }
})

export const viewState = state => state.screens.createconversation
export const viewSelector = createViewSelector(viewState)

export const message = viewSelector('message', '')
export const recipients = viewSelector('recipients', List())
export const isLoading = viewSelector('isLoading', false)
export const hasLoaded = createSelector(
  [
    ProjectSelectors.hasListLoaded,
    ContactSelectors.hasRecentContactsListLoaded,
    isLoading
  ],
  (hasProjectLoaded, hasContactLoaded, isLoading) =>
    !isLoading && hasProjectLoaded && hasContactLoaded
)
export const contacts = createSelector(
  [ContactSelectors.contacts, ContactSelectors.recentContactsList],
  (contacts, contactsList) =>
    contactsList.reduce(
      (out, id) => out.push(contacts.getIn([id, 'name'])),
      List()
    )
)
export const selectedProjects = viewSelector('selectedProjects', List())

export const filteredProjectList = createSelector(
  [ProjectSelectors.projectsList, ProjectSelectors.projects],
  (projectList, projects) =>
    projectList.filter(id => !projects.getIn([id, 'isSub']))
)

export const selectors = {
  message,
  contacts,
  selectedProjects,
  hasLoaded,
  recipients,
  projects: ProjectSelectors.projects,
  projectList: filteredProjectList
}
