import React from 'react'
import { shallow } from 'enzyme'
import { InboxSchema } from 'resources/inbox'
import ConversationRow from '../components/ConversationRow'
import { fromJS, List } from 'immutable'
import { TouchableOpacity, Text } from 'react-native'
import ConversationBadge from 'components/ConversationBadge'
import TimeFormatter from 'components/TimeFormatter'

const noop = jest.fn()
const messageCreatedAt = new Date(2018, 2, 2, 16, 12, 11)
const inbox = fromJS({
  '12': new InboxSchema({
    createdAt: messageCreatedAt,
    partecipants: fromJS(['44', '56']),
    project: '33',
    conversation: '453',
    room: 'abc',
    latestMessage: 'latest message'
  })
})
const projects = fromJS({ '33': { name: 'asdfoo' } })
const users = fromJS({
  '44': {
    name: 'Daniele'
  },
  '56': {
    name: 'Alex'
  }
})
const inboxList = new List()

function renderHelper (props) {
  return shallow(
    <ConversationRow
      inbox={inbox}
      id='12'
      projects={projects}
      toConversation={noop}
      users={users}
      inboxList={inboxList}
      {...props}
    />
  )
}

describe('Inbox ConversationRow', () => {
  it('is wrapped in a touchableopacity', () => {
    const toConversation = jest.fn()
    const node = renderHelper({ toConversation })
    expect(node.type()).toBe(TouchableOpacity)
    expect(node.find(TouchableOpacity)).toHaveLength(1)
    node.find(TouchableOpacity).prop('onPress')()
    expect(toConversation).toHaveBeenCalledWith({
      conversationId: '453',
      roomId: 'abc'
    })
  })
  it('render a ConversationBadge', () => {
    const inboxUnread = fromJS({
      '12': new InboxSchema({
        createdAt: new Date(),
        partecipants: fromJS(['44', '56']),
        project: '33',
        conversation: '453',
        unread: true
      })
    })
    const nodeWithUnread = renderHelper({ inbox: inboxUnread })
    const nodeWithRead = renderHelper({ inbox })
    expect(inbox.getIn(['12', 'unread'])).toBe(null)
    expect(inboxUnread.getIn(['12', 'unread'])).toBe(true)
    expect(nodeWithUnread.find(ConversationBadge).exists()).toBeTruthy()
    expect(nodeWithRead.find(ConversationBadge).exists()).toBeTruthy()
    expect(nodeWithUnread.find(ConversationBadge).prop('unread')).toBeTruthy()
    expect(nodeWithRead.find(ConversationBadge).prop('unread')).toBeFalsy()
  })
  it('render a text with partecipants', () => {
    const node = renderHelper()
    expect(
      node
        .find(Text)
        .map(text => text.prop('children'))
        .includes('Daniele, Alex')
    ).toBeTruthy()
  })
  it('render a TimeFormatter', () => {
    const node = renderHelper()
    expect(node.find(TimeFormatter).prop('date')).toBe(messageCreatedAt)
  })
  it('render a Text with latest message', () => {
    const node = renderHelper()
    expect(
      node
        .find(Text)
        .map(text => text.prop('children'))
        .includes('latest message')
    ).toBeTruthy()
  })
  it('defaults latest message to "no message"', () => {
    const node = renderHelper()
    expect(
      node
        .find(Text)
        .map(text => text.prop('children'))
        .includes('latest message')
    ).toBeTruthy()
  })
  it('render a text with the project name', () => {
    const node = renderHelper()
    expect(
      node
        .find(Text)
        .map(text => text.prop('children'))
        .includes(projects.getIn(['33', 'name']))
    ).toBeTruthy()
  })
})
