import createStoreForTesting from 'test/createstore'
import SCREENS from 'routes/screens'
import { UserActions } from 'resources/user'
import { CONSTANTS, actions, selectors } from '../module'
import { InboxSelectors } from 'resources/inbox'
import { API_URL } from '../../../config'
import { List } from 'immutable'
import inboxRecord from 'test/records/inbox.json'
import { NavigationActions } from 'react-navigation'

const USER_ID = '379'
const PROJECT_ID = 'foobar'

describe('Inbox module', () => {
  describe('initialize', () => {
    it('should reset search and project filter', async () => {
      const store = createStoreForTesting()
      store.dispatch(actions.initialize())
      expect(store.spyActions.findAction(CONSTANTS.SET_SEARCH)).toEqual({
        payload: '',
        type: CONSTANTS.SET_SEARCH
      })
      expect(store.spyActions.findAction(CONSTANTS.SET_PROJECT_FILTER)).toEqual(
        {
          payload: '0',
          type: CONSTANTS.SET_PROJECT_FILTER
        }
      )
    })
  })
  describe('setSearch', () => {
    it('should set the search needle', () => {
      const store = createStoreForTesting()
      store.dispatch(actions.setSearch(PROJECT_ID))
      expect(selectors.search(store.getState())).toBe(PROJECT_ID)
    })
  })
  describe('setProject', () => {
    it('should set the project', () => {
      const store = createStoreForTesting()
      store.dispatch(actions.setProject(PROJECT_ID))
      expect(selectors.projectId(store.getState())).toBe(PROJECT_ID)
    })
  })
  describe('setProjectFilter', async () => {
    it('should set the project and fetch conversations from api', async () => {
      const store = createStoreForTesting()
      store.dispatch(UserActions.setCurrentUser(USER_ID))
      store.addResponse(inboxRecord)
      await store.dispatch(actions.setProjectFilter(PROJECT_ID))
      expect(selectors.projectId(store.getState())).toBe(PROJECT_ID)
      expect(fetch).toHaveBeenCalledWith(
        `${API_URL}/message/inbox/${PROJECT_ID}/${USER_ID}`,
        {
          headers: {
            Accept: 'application/json',
            'Content-Type': 'application/json'
          },
          method: 'GET'
        }
      )
    })
  })
  describe('changeProjectFilter', () => {
    it('should route to project selector', () => {
      const store = createStoreForTesting()
      store.dispatch(actions.changeProjectFilter())
      expect(store.spyActions.findAction(NavigationActions.NAVIGATE)).toEqual({
        type: NavigationActions.NAVIGATE,
        routeName: SCREENS.SELECT_PROJECT,
        params: {
          displayAll: true,
          selected: ['0'],
          singleSelection: true,
          save: actions.acceptSelectProject
        }
      })
    })
  })
  describe('toConversation', () => {
    it('should go to selected conversation', () => {
      const store = createStoreForTesting()
      store.dispatch(
        actions.toConversation({ conversationId: '1234', roomId: 'abc' })
      )
      const action = store.spyActions.findAction(NavigationActions.NAVIGATE)
      expect(action.type).toBe(NavigationActions.NAVIGATE)
      expect(action.routeName).toBe(SCREENS.CONVERSATION)
      expect(action.params.conversationId).toBe('1234')
      expect(action.params.roomId).toBe('abc')
    })
  })

  describe('processedInbox', () => {
    it('should remove current user from partecipants', async () => {
      const store = createStoreForTesting()
      store.dispatch(UserActions.setCurrentUser(USER_ID))
      store.addResponse(inboxRecord)
      await store.dispatch(actions.setProjectFilter(PROJECT_ID))

      const naturalInbox = InboxSelectors.inbox(store.getState())
      naturalInbox.forEach(message => {
        expect(message.partecipants.includes(USER_ID)).toBeTruthy()
      })

      const processedInbox = selectors.processedInbox(store.getState())
      processedInbox.forEach(message => {
        expect(message.partecipants.includes(USER_ID)).toBeFalsy()
      })
    })
    it('should map to inbox', () => {
      expect(selectors.inbox).toBe(selectors.processedInbox)
    })
  })

  describe('canViewDetails', () => {
    it('can only view details if a project is selected', () => {
      const store = createStoreForTesting()
      expect(selectors.projectId(store.getState())).toBe('0')
      expect(selectors.canViewDetails(store.getState())).toBeFalsy()
      store.dispatch(actions.setProject(PROJECT_ID))
      expect(selectors.projectId(store.getState())).toBe(PROJECT_ID)
      expect(selectors.canViewDetails(store.getState())).toBeTruthy()
    })
  })

  describe('filteredInboxList', () => {
    let store
    beforeEach(async () => {
      store = createStoreForTesting()
      store.dispatch(UserActions.setCurrentUser(USER_ID))
      store.addResponse(inboxRecord)
      await store.dispatch(actions.setProjectFilter(PROJECT_ID))
    })
    // we can't test the content, due to ID of messages changing everytime
    it('should filter conversation by partecipants', () => {
      expect(selectors.inboxList(store.getState()).size).toBe(2)
      store.dispatch(actions.setSearch('test'))
      expect(selectors.inboxList(store.getState())).toEqual(List())
      expect(selectors.inboxList(store.getState()).size).toBe(0)
      store.dispatch(actions.setSearch('Jackie'))
      expect(typeof selectors.inboxList(store.getState())).toBe(typeof List())
      expect(selectors.inboxList(store.getState()).size).toBe(1)
      store.dispatch(actions.setSearch('Somewhere'))
      expect(typeof selectors.inboxList(store.getState())).toBe(typeof List())
      expect(selectors.inboxList(store.getState()).size).toBe(1)
    })
    it('should not be case sensitive', () => {
      expect(selectors.inboxList(store.getState()).size).toBe(2)
      store.dispatch(actions.setSearch('tEsT'))
      expect(selectors.inboxList(store.getState())).toEqual(List())
      expect(selectors.inboxList(store.getState()).size).toBe(0)
      store.dispatch(actions.setSearch('jackie'))
      expect(typeof selectors.inboxList(store.getState())).toBe(typeof List())
      expect(selectors.inboxList(store.getState()).size).toBe(1)
      store.dispatch(actions.setSearch('somewhere'))
      expect(typeof selectors.inboxList(store.getState())).toBe(typeof List())
      expect(selectors.inboxList(store.getState()).size).toBe(1)
    })
  })

  describe('isEmpty', () => {
    let store
    beforeEach(async () => {
      store = createStoreForTesting()
      store.dispatch(UserActions.setCurrentUser(USER_ID))
      store.addResponse(inboxRecord)
      await store.dispatch(actions.setProjectFilter(PROJECT_ID))
    })
    describe('filtered', () => {
      it('should return true if we have no conversations', () => {
        const store = createStoreForTesting()
        store.dispatch(actions.setSearch('tEsT'))
        expect(selectors.inboxList(store.getState()).size).toBe(0)
        expect(selectors.isEmpty(store.getState())).toBeTruthy()
      })
      it('should return true if conversations are there but filtered out', () => {
        store.dispatch(actions.setSearch('tEsT'))
        expect(selectors.inboxList(store.getState()).size).toBe(0)
        expect(selectors.isEmpty(store.getState())).toBeTruthy()
        store.dispatch(actions.setSearch('jackie'))
        expect(selectors.inboxList(store.getState()).size).toBe(1)
        expect(selectors.isEmpty(store.getState())).toBeFalsy()
      })
    })
    describe('unfiltered', () => {
      it('should return true if we have no conversations', () => {
        const store = createStoreForTesting()
        expect(selectors.inboxList(store.getState()).size).toBe(0)
        expect(selectors.isEmpty(store.getState())).toBeTruthy()
      })
      it('should return false if conversations are there', () => {
        expect(selectors.inboxList(store.getState()).size).toBe(2)
        expect(selectors.isEmpty(store.getState())).toBeFalsy()
      })
    })
  })
})
