// @flow
import { Record, List } from 'immutable'
import { createSelector } from 'reselect'
import { createAction } from 'redux-actions'
import createConstants from 'utils/createconstants'
import createReducer from 'utils/createreducer'
import createViewSelector from 'utils/createviewselector'
import { InboxActions, InboxSelectors } from 'resources/inbox'
import { UserSelectors } from 'resources/user'
import { ProjectSelectors } from 'resources/project'
import PushNotification from 'react-native-push-notification'
import { RouteActions, SCREENS } from 'routes'

export const CONSTANTS = createConstants('VIEW_INBOX', {
  SET_SEARCH: null,
  SET_PROJECT: null,
  SET_PROJECT_FILTER: null,
  SET_DISPLAY_PROJECT_FILTER: null
})

export const actions = {
  initialize ({ projectId, fromProjectInfo } = {}) {
    return dispatch => {
      dispatch(actions.setSearch(''))
      dispatch(
        RouteActions.setParams({
          createConversation: () => dispatch(actions.toCreateConversation()),
          displayBack: fromProjectInfo || false
        })
      )
      dispatch(actions.setDisplayProjectFilter(!fromProjectInfo))
      PushNotification.setApplicationIconBadgeNumber(0)
      return dispatch(actions.setProjectFilter(projectId || '0'))
    }
  },
  setDisplayProjectFilter: createAction(CONSTANTS.SET_DISPLAY_PROJECT_FILTER),
  setSearch: createAction(CONSTANTS.SET_SEARCH),
  setProject: createAction(CONSTANTS.SET_PROJECT),
  setProjectFilter (projectId) {
    return dispatch => {
      dispatch({ type: CONSTANTS.SET_PROJECT_FILTER, payload: projectId })
      dispatch(actions.setProject(projectId))
      return dispatch(InboxActions.fetchList({ projectId }))
    }
  },
  changeProjectFilter () {
    return (dispatch, getState) => {
      const projectId = selectors.projectId(getState())
      return dispatch(
        RouteActions.navigate(SCREENS.SELECT_PROJECT, {
          save: actions.acceptSelectProject,
          displayAll: true,
          singleSelection: true,
          selected: [projectId]
        })
      )
    }
  },
  acceptSelectProject (selection) {
    return actions.setProjectFilter(selection.get(0))
  },
  refreshInbox () {
    return (dispatch, getState) => {
      const projectId = selectors.projectId(getState())
      dispatch(InboxActions.fetchList({ projectId }))
    }
  },
  toConversation ({ conversationId, roomId }) {
    return (dispatch, getState) => {
      dispatch(
        RouteActions.navigate(SCREENS.CONVERSATION, {
          conversationId,
          roomId,
          refreshInbox: actions.refreshInbox
        })
      )
    }
  },
  toCreateConversation () {
    return (dispatch, getState) => {
      const projectId = selectors.projectId(getState())
      dispatch(RouteActions.navigate(SCREENS.CREATE_CONVERSATION), {
        projectId
      })
    }
  },
  toProjectInfo () {
    return (dispatch, getState) => {
      const projectId = selectors.projectId(getState())
      dispatch(
        RouteActions.navigate(SCREENS.PROJECT_INFO, {
          projectId
        })
      )
    }
  }
}

const InboxState = Record({
  projectId: '0',
  search: '',
  displayProjectFilter: true
})

export const reducer = createReducer(new InboxState(), {
  [CONSTANTS.SET_SEARCH] (state, { payload }) {
    return state.set('search', payload)
  },
  [CONSTANTS.SET_PROJECT] (state, { payload }) {
    return state.set('projectId', payload)
  },
  [CONSTANTS.SET_DISPLAY_PROJECT_FILTER] (state, { payload }) {
    return state.set('displayProjectFilter', payload)
  }
})

export const viewState = state => state.screens.inbox
export const viewSelector = createViewSelector(viewState)
export const search = viewSelector('search', '')

export const lowercaseSearch = createSelector([search], search =>
  search.toLowerCase()
)

export const filteredInboxList = createSelector(
  [
    InboxSelectors.inbox,
    InboxSelectors.inboxList,
    UserSelectors.users,
    ProjectSelectors.projects,
    lowercaseSearch,
    UserSelectors.currentUserId
  ],
  (inboxes, inboxList, users, projects, search, currentUserId) =>
    inboxList.filter(inboxId => {
      const inbox = inboxes.get(inboxId)
      const userNames = inbox
        .get('partecipants')
        .map(userId => users.getIn([userId, 'name']))
      const matchUsers = userNames.filter(
        userName => userName.toLowerCase().indexOf(search) !== -1
      )
      const project = projects.get(inbox.get('project'))
      const hasPartecipants =
        inbox
          .get('partecipants', List())
          .filter(userId => userId !== currentUserId).size > 0
      const matchProject =
        project
          .get('name')
          .toLowerCase()
          .indexOf(search) !== -1
      return (matchUsers.size > 0 || matchProject) && hasPartecipants
    })
)

export const processedInbox = createSelector(
  [InboxSelectors.inbox, UserSelectors.currentUserId],
  (inbox, currentUserId) =>
    inbox.map(message => {
      const partecipants = message
        .get('partecipants', List())
        .filter(userId => userId !== currentUserId)
      return message.set('partecipants', partecipants)
    })
)

export const isEmpty = createSelector(
  [filteredInboxList, search],
  (list, search) => !list.size
)

const projectId = viewSelector('projectId')
const canViewDetails = createSelector(
  [projectId],
  projectId => projectId !== '0'
)

const displayProjectFilter = viewSelector('displayProjectFilter', true)

export const selectors = {
  projectId,
  canViewDetails,
  displayProjectFilter,
  isLoading: InboxSelectors.isListLoading,
  users: UserSelectors.users,
  projects: ProjectSelectors.projects,
  search,
  isEmpty,
  processedInbox,
  inbox: processedInbox,
  inboxList: filteredInboxList
}
