import { AsyncStorage, Keyboard } from 'react-native'
import createStoreForTesting from 'test/createstore'
import loginRecord from 'test/records/login.json'
import { API_URL, USERPHONE_KEY } from '../../../config'
import { NavigationActions } from 'react-navigation'
import SCREENS from 'routes/screens'
import {
  CONSTANTS,
  actions,
  selectors,
  viewState,
  LoginState,
  VIDEO_STATES
} from '../module'

describe('Login module', () => {
  beforeEach(() => AsyncStorage.clear())
  describe('initialize', () => {
    it('should load from AsyncStorage the last known phone number', async () => {
      const store = createStoreForTesting()
      await AsyncStorage.setItem(USERPHONE_KEY, '112233')
      await store.dispatch(actions.initialize())
      expect(selectors.phoneNumber(store.getState())).toBe('112233')
    })
    it('should not set phone number if nothing is available in AsyncStorage', async () => {
      const store = createStoreForTesting()
      expect(await AsyncStorage.getItem(USERPHONE_KEY)).toBe(null)
      await store.dispatch(actions.initialize())
      expect(selectors.phoneNumber(store.getState())).toBe('')
    })
  })

  describe('setPhone', () => {
    it('should set the phone number', () => {
      const store = createStoreForTesting()
      expect(selectors.phoneNumber(store.getState())).toBe('')
      store.dispatch(actions.setPhoneNumber('34'))
      expect(selectors.phoneNumber(store.getState())).toBe('34')
    })
    it('should reset the errors', () => {
      const store = createStoreForTesting()
      expect(selectors.errorMessage(store.getState())).toBe(null)
      store.dispatch(actions.setErrorMessage('error message'))
      expect(selectors.errorMessage(store.getState())).toBe('error message')
      store.dispatch(actions.setPhoneNumber('34'))
      expect(selectors.errorMessage(store.getState())).toBe(null)
    })
  })

  describe('resetState', () => {
    it('should reset state', () => {
      const store = createStoreForTesting()
      expect(viewState(store.getState())).toEqual(new LoginState())
      store.dispatch(actions.setErrorMessage('error'))
      expect(viewState(store.getState())).not.toEqual(new LoginState())
      store.dispatch(actions.resetState())
      expect(viewState(store.getState())).toEqual(new LoginState())
    })
  })
  describe('setSubmitting', () => {
    it('should set submit state', () => {
      const store = createStoreForTesting()
      expect(selectors.isSubmitting(store.getState())).toBe(false)
      store.dispatch(actions.setSubmitting(true))
      expect(selectors.isSubmitting(store.getState())).toBe(true)
    })
  })
  describe('setPlaying', () => {
    it('should set playing state', () => {
      const store = createStoreForTesting()
      expect(selectors.isPlaying(store.getState())).toBe(false)
      store.dispatch(actions.setPlaying(true))
      expect(selectors.isPlaying(store.getState())).toBe(true)
    })
  })
  describe('setVideoState', () => {
    it('should set playing state for PAUSED state to false', () => {
      const store = createStoreForTesting()
      expect(selectors.isPlaying(store.getState())).toBe(false)
      store.dispatch(actions.setVideoState(VIDEO_STATES.PAUSED))
      expect(selectors.isPlaying(store.getState())).toBe(false)
    })
    it('should set playing state for ENDED state to false', () => {
      const store = createStoreForTesting()
      expect(selectors.isPlaying(store.getState())).toBe(false)
      store.dispatch(actions.setVideoState(VIDEO_STATES.ENDED))
      expect(selectors.isPlaying(store.getState())).toBe(false)
    })
    it('should set playing state for PLAYING state to true', () => {
      const store = createStoreForTesting()
      expect(selectors.isPlaying(store.getState())).toBe(false)
      store.dispatch(actions.setVideoState(VIDEO_STATES.PLAYING))
      expect(selectors.isPlaying(store.getState())).toBe(true)
    })
  })
  describe('setErrorMessage', () => {
    it('should set the error message', () => {
      const store = createStoreForTesting()
      expect(selectors.errorMessage(store.getState())).toBe(null)
      store.dispatch(actions.setErrorMessage('error message'))
      expect(selectors.errorMessage(store.getState())).toBe('error message')
    })
  })
  describe('sendPin', () => {
    it('should validate the phone number', async () => {
      const store = createStoreForTesting()
      store.dispatch(actions.setPhoneNumber('123'))
      await store.dispatch(actions.sendPin())
      expect(selectors.errorMessage(store.getState())).toBe(
        'Invalid phone number'
      )
    })
    it('should call the api to get the verification code', async () => {
      const store = createStoreForTesting()
      store.dispatch(actions.setPhoneNumber('1234567890'))
      store.addResponse(loginRecord)
      await store.dispatch(actions.sendPin())
      expect(fetch).toHaveBeenCalledWith(`${API_URL}/user`, {
        body: JSON.stringify({ cell: '1234567890' }),
        headers: {
          Accept: 'application/json',
          'Content-Type': 'application/json'
        },
        method: 'POST'
      })
    })
    it('on success it should push to enter code screen passing the phone number', async () => {
      const store = createStoreForTesting()
      store.dispatch(actions.setPhoneNumber('1234567890'))
      store.addResponse(loginRecord)
      await store.dispatch(actions.sendPin())
      const action = store.spyActions.findAction(NavigationActions.NAVIGATE)
      expect(action.routeName).toBe(SCREENS.VERIFY_PIN)
      expect(action.params).toEqual({ phoneNumber: '1234567890' })
    })
    it('on error it should set the error', async () => {
      const store = createStoreForTesting()
      store.dispatch(actions.setPhoneNumber('1234567890'))
      store.addReject()
      await store.dispatch(actions.sendPin())
      expect(selectors.errorMessage(store.getState())).toBe(
        "There's been an error requesting the PIN"
      )
    })
    it('should dismiss the keyboard', async () => {
      const store = createStoreForTesting()
      Keyboard.clearCallCount()
      expect(Keyboard.callCount()).toBe(0)
      store.dispatch(actions.setPhoneNumber('1234567890'))
      store.addResponse(loginRecord)
      await store.dispatch(actions.sendPin())
      expect(Keyboard.callCount()).toBe(1)
    })
    it('should dismiss the video', async () => {
      const store = createStoreForTesting()
      store.dispatch(actions.setPhoneNumber('1234567890'))
      store.addResponse(loginRecord)
      await store.dispatch(actions.sendPin())
      const videoPlaying = store.spyActions.findAction(
        CONSTANTS.SET_VIDEOPLAYING
      )
      expect(videoPlaying).not.toBe(undefined)
      expect(videoPlaying.payload).toBe(false)
    })
  })
})
