// @flow
import { Record, List } from 'immutable'
import { Alert } from 'react-native'
import RNGooglePlacePicker from 'react-native-google-place-picker'
import createReducer from 'utils/createreducer'
import createConstants from 'utils/createconstants'
import createViewSelector from 'utils/createviewselector'
import Geocoder from 'react-native-geocoder'
import * as RouteActions from 'routes/actions'
import { ProjectActions, ProjectSelectors } from 'resources/project'
import { ContactApi, ContactSelectors } from 'resources/contact'
import { UserSelectors } from 'resources/user'
import { ProjectInfoActions } from 'screens/ProjectInfo'
import sanitizeLocation from 'utils/sanitizelocation'
import { GOOGLE_API_KEY } from '../../config'

Geocoder.fallbackToGoogle(GOOGLE_API_KEY)

export const CONSTANTS = createConstants('SCREEN_PROJECT_EDIT', {
  SET_NAME: null,
  RESET: null,
  SET_LOCATION: null,
  SET_LOADING: null,
  SET_PROJECT_ID: null,
  SET_PROJECT_NUMBER: null,
  SET_CONTACT_NUMBER: null,
  SET_CONTACT_NAME: null,
  SET_CONTACTS: null
})

export const actions = {
  initialize ({ projectId }) {
    return async (dispatch, getState) => {
      dispatch(actions.reset())
      dispatch(actions.setProjectId(projectId))
      await dispatch(ProjectActions.fetchInfo({ projectId }))
      const project = selectors.project(getState())
      dispatch(actions.setName(project.get('name')))
      dispatch(actions.setProjectNumber(project.get('phoneNumber')))
      dispatch(
        RouteActions.setParams({
          saveProject: () => dispatch(actions.saveProject())
        })
      )
      await dispatch(ProjectActions.fetchInfo({ projectId }))
      dispatch(actions.setName(project.get('name')))
      dispatch(
        actions.setLocation({
          latitude: project.get('lat'),
          longitude: project.get('long'),
          address: sanitizeLocation(project.get('location'))
        })
      )
      const contacts = ContactSelectors.contacts(getState())
      const contactList = project.get('contacts') || List()
      const mappedContacts = contactList.map(id => contacts.get(id))
      dispatch(actions.setContacts(mappedContacts))
    }
  },
  setProjectId (projectId) {
    return {
      type: CONSTANTS.SET_PROJECT_ID,
      payload: projectId
    }
  },
  reset () {
    return {
      type: CONSTANTS.RESET
    }
  },
  setLoading (loading) {
    return {
      type: CONSTANTS.SET_LOADING,
      payload: loading
    }
  },
  setName (name) {
    return {
      type: CONSTANTS.SET_NAME,
      payload: name
    }
  },
  setLocation ({ latitude, longitude, address }) {
    return {
      type: CONSTANTS.SET_LOCATION,
      payload: {
        latitude,
        longitude,
        address
      }
    }
  },
  setProjectNumber (number) {
    return {
      type: CONSTANTS.SET_PROJECT_NUMBER,
      payload: number
    }
  },
  setContactName (index, name) {
    return {
      type: CONSTANTS.SET_CONTACT_NAME,
      payload: {
        index,
        name
      }
    }
  },
  setContactNumber (index, number) {
    return {
      type: CONSTANTS.SET_CONTACT_NUMBER,
      payload: {
        index,
        number
      }
    }
  },

  setContacts (contacts) {
    return {
      type: CONSTANTS.SET_CONTACTS,
      payload: contacts
    }
  },

  selectLocation () {
    return dispatch => {
      RNGooglePlacePicker.show(async response => {
        if (response.didCancel || response.error) {
          return
        }
        dispatch(actions.setLoading(true))
        const geocode = await Geocoder.geocodePosition({
          lat: response.latitude,
          lng: response.longitude
        })
        dispatch(
          actions.setLocation({
            latitude: response.latitude,
            longitude: response.longitude,
            address: geocode.length ? response.address : 'Unknown Address'
          })
        )
        dispatch(actions.setLoading(false))
      })
    }
  },
  saveProject () {
    return async (dispatch, getState) => {
      const state = getState()
      const name = selectors.name(state)
      const address = selectors.address(state)
      const lat = selectors.lat(state)
      const long = selectors.long(state)
      const contacts = selectors.contacts(state)
      const projectId = selectors.projectId(state)
      const userId = UserSelectors.currentUserId(state)
      const projectNumber = selectors.projectNumber(state)

      if (!address || !name) {
        Alert.alert('Nailit!', 'Fill in the details')
        return
      }
      try {
        dispatch(actions.setLoading(true))
        // add contacts first
        const apiContacts = (await Promise.all(
          contacts.toJS().map(contact =>
            ContactApi.addContact({
              id: contact.id,
              contactId: contact.id,
              name: contact.name,
              phone: contact.phone,
              email: '',
              projectId,
              userId
            })
          )
        )).map(response => response.data)

        // save project
        await dispatch(
          ProjectActions.editProject({
            projectId,
            name,
            address,
            lat,
            long,
            phoneNumber: projectNumber,
            contacts: apiContacts
          })
        )
        await dispatch(ProjectActions.fetchList())
        await dispatch(ProjectActions.fetchInfo({ projectId }))
        dispatch(RouteActions.back())
        setTimeout(() => {
          dispatch(ProjectInfoActions.refreshTitle())
        }, 200)
      } catch (error) {
        console.log('edit project error', error)
        Alert.alert('Nailit!', 'Could not edit project')
      } finally {
        dispatch(actions.setLoading(false))
      }
    }
  }
}

export const ProjectEditState = Record({
  name: '',
  lat: 0,
  contacts: List(),
  isLoading: false,
  long: 0,
  address: '',
  projectNumber: '',
  projectId: null
})

export const reducer = createReducer(new ProjectEditState(), {
  [CONSTANTS.RESET]: () => new ProjectEditState(),
  [CONSTANTS.SET_NAME]: (state, { payload }) => state.set('name', payload),
  [CONSTANTS.SET_CONTACTS]: (state, { payload }) =>
    state.set('contacts', payload),
  [CONSTANTS.SET_CONTACT_NAME]: (state, { payload }) =>
    state.setIn(['contacts', payload.index, 'name'], payload.name),
  [CONSTANTS.SET_CONTACT_NUMBER]: (state, { payload }) =>
    state.setIn(['contacts', payload.index, 'phone'], payload.number),
  [CONSTANTS.SET_PROJECT_ID]: (state, { payload }) =>
    state.set('projectId', payload),
  [CONSTANTS.SET_PROJECT_NUMBER]: (state, { payload }) =>
    state.set('projectNumber', payload),
  [CONSTANTS.SET_LOADING]: (state, { payload }) =>
    state.set('isLoading', payload),
  [CONSTANTS.SET_LOCATION]: (state, { payload }) => {
    const { latitude, longitude, address } = payload
    return state
      .set('lat', latitude)
      .set('long', longitude)
      .set('address', address)
  }
})

export const viewState = state => state.screens.projectedit
export const viewSelector = createViewSelector(viewState)
export const name = viewSelector('name')
export const long = viewSelector('long')
export const lat = viewSelector('lat')
export const address = viewSelector('address')
export const isLoading = viewSelector('isLoading')
export const projectId = viewSelector('projectId')
export const projectName = viewSelector('projectName')
export const projectNumber = viewSelector('projectNumber')
export const project = ProjectSelectors.getProject(projectId)
export const contacts = viewSelector('contacts', List())

export const selectors = {
  name,
  address,
  long,
  lat,
  isLoading,
  projectId,
  project,
  contacts,
  projectNumber,
  projectName
}
