import React from 'react'
import { shallow } from 'enzyme'
import { Map, List } from 'immutable'
import { ScrollView } from 'react-native'
import { ProjectSchema } from 'resources/project'
import { SolidButton } from 'components/Button'
import ProjectInfo from '../ProjectInfo'
import ProjectMap from '../components/ProjectMap'
import ButtonBar from '../components/ButtonBar'
import SubsView from '../components/SubsView'
import ProjectTabBar from '../components/ProjectTabBar'
import ProjectInfoLoader from '../components/ProjectInfoLoader'

function renderHelper ({ projectDetails = {}, ...props } = {}) {
  const project = new ProjectSchema({
    lat: 25,
    long: 66,
    address: 'Route 88',
    ...projectDetails
  })
  return shallow(
    <ProjectInfo
      addSub={jest.fn()}
      announce={jest.fn()}
      project={project}
      contacts={new Map()}
      hasLoaded
      isLoading={false}
      hasPhoneNumber
      onDelete={jest.fn()}
      toMessage={jest.fn()}
      toCall={jest.fn()}
      toDirections={jest.fn()}
      toConversation={jest.fn()}
      toCaptureImages={jest.fn()}
      toRecord={jest.fn()}
      toInbox={jest.fn()}
      toAttachFiles={jest.fn()}
      {...props}
    />
  )
}

describe('Project Info', () => {
  it.skip('should not change by accident', () => {
    expect(renderHelper()).toMatchSnapshot()
  })
  it('should render a loader if data is not loaded', () => {
    const notLoadedNode = renderHelper({ hasLoaded: false })
    const loadedNode = renderHelper({ hasLoaded: true })
    expect(notLoadedNode).toHaveLength(1)
    expect(notLoadedNode.find(ProjectInfoLoader).exists()).toBe(true)
    expect(loadedNode).toHaveLength(1)
    expect(loadedNode.find(ProjectInfoLoader).exists()).toBe(false)
  })
  it('should render a button bar passing props down', () => {
    const toMessage = jest.fn()
    const toCall = jest.fn()
    const toDirections = jest.fn()
    const node = renderHelper({
      hasPhoneNumber: true,
      toMessage,
      toCall,
      toDirections
    })
    const buttonBar = node.find(ButtonBar)
    expect(buttonBar.exists()).toBe(true)
    expect(buttonBar.prop('hasPhoneNumber')).toBe(true)
    expect(buttonBar.prop('toMessage')).toBe(toMessage)
    expect(buttonBar.prop('toCall')).toBe(toCall)
    expect(buttonBar.prop('toDirections')).toBe(toDirections)
  })
  it('should render a project map passing the project down', () => {
    const project = new ProjectSchema({
      lat: 25,
      long: 66,
      address: 'Route 88'
    })
    const node = renderHelper({
      project
    })
    const projectMap = node.find(ProjectMap)
    expect(projectMap.exists()).toBe(true)
    expect(projectMap.prop('project')).toBe(project)
  })
  it('should render a scrollview with 2 buttons and SubsView in it', () => {
    const announce = jest.fn()
    const addSub = jest.fn()
    const node = renderHelper({
      announce,
      addSub
    })
    const scrollView = node.find(ScrollView)
    expect(scrollView.exists()).toBe(true)
    const subView = scrollView.find(SubsView)
    expect(subView.exists()).toBe(true)
    const buttons = scrollView.find(SolidButton)
    expect(buttons).toHaveLength(2)
    const buttonText = buttons.map(button => button.children().text())
    const buttonPress = buttons.map(button => button.prop('onPress'))
    expect(buttonText).toMatchSnapshot()
    expect(buttonPress.includes(announce)).toBe(true)
    expect(buttonPress.includes(addSub)).toBe(true)
  })
  it('should pass the expected props to SubView', () => {
    const tasks = new Map()
    const subs = new Map()
    const tasksSubList = new List()
    const toConversation = jest.fn()
    const node = renderHelper({
      tasks,
      subs,
      tasksSubList,
      toConversation
    })
    const subView = node.find(SubsView)
    expect(subView.exists()).toBe(true)
    expect(subView.prop('tasks')).toBe(tasks)
    expect(subView.prop('subs')).toBe(subs)
    expect(subView.prop('tasksSubList')).toBe(tasksSubList)
    expect(subView.prop('toConversation')).toBe(toConversation)
  })
  it('should render a ProjectTabBar with 3 tabs', () => {
    const toCaptureImages = jest.fn()
    const toRecord = jest.fn()
    const toAttachFiles = jest.fn()
    const toInbox = jest.fn()
    const node = renderHelper({
      toCaptureImages,
      toRecord,
      toAttachFiles,
      toInbox
    })
    const tabBar = node.find(ProjectTabBar)
    expect(tabBar.exists()).toBe(true)
    const tabs = tabBar.prop('tabs')
    expect(tabs).toHaveLength(3)
    expect(tabs).toMatchSnapshot()
    const tabsFn = tabs.map(tab => tab.onPress)
    expect(tabsFn.includes(toCaptureImages)).toBe(true)
    // expect(tabsFn.includes(toRecord)).toBe(true)
    expect(tabsFn.includes(toAttachFiles)).toBe(true)
    expect(tabsFn.includes(toInbox)).toBe(true)
  })
})
