// @flow
import { Map, List, Record, fromJS } from 'immutable'
import { Alert } from 'react-native'
import createReducer from 'utils/createreducer'
import createConstants from 'utils/createconstants'
import { createSelector } from 'reselect'
import { ProjectActions, ProjectSelectors } from 'resources/project'
import { TaskSelectors, TaskActions } from 'resources/task'
import { InboxActions, InboxSelectors } from 'resources/inbox'
import { UserSelectors } from 'resources/user'
import { ConversationActions } from 'resources/conversation'
import { ContactSelectors } from 'resources/contact'
import createViewSelector from 'utils/createviewselector'
import { RouteActions } from 'routes'
import comms from 'vendor/react-native-communications'
import { showLocation } from 'vendor/react-native-map-link'
import { ModalActions } from 'components/Modal'
import SCREENS from 'routes/screens'
import sanitizeLocation from 'utils/sanitizelocation'

export const CONSTANTS = createConstants('SCREEN_PROJECT_INFO', {
  SET_PROJECT_ID: null,
  SET_LOADING: null
})

export const actions = {
  initialize ({ projectId, isSub }) {
    return async (dispatch, getState) => {
      dispatch(actions.setProjectId(projectId))
      dispatch(TaskActions.reset())
      await dispatch(ProjectActions.fetchInfo({ projectId }))
      const project = selectors.project(getState())
      dispatch(
        RouteActions.setParams({
          editProject: () => dispatch(actions.toEditProject()),
          isSub: project.get('isSub')
        })
      )

      await dispatch(TaskActions.fetchForProject({ projectId }))
      dispatch(RouteActions.setTitle(project.get('name')))
    }
  },
  setProjectId (projectId) {
    return {
      type: CONSTANTS.SET_PROJECT_ID,
      payload: projectId
    }
  },
  refreshTitle () {
    return async (dispatch, getState) => {
      const project = selectors.project(getState())
      dispatch(RouteActions.setTitle(project.get('name')))
      dispatch(
        RouteActions.setParams({
          editProject: () => dispatch(actions.toEditProject())
        })
      )
    }
  },
  toMessage () {
    return async (dispatch, getState) => {
      const project = ProjectSelectors.getProject(projectId)(getState())
      const contacts = ContactSelectors.contacts(getState())
      const contact = contacts.get(project.getIn(['contacts', 0]))
      if (contact && contact.get('phone') && contact.get('phone').length > 1) {
        try {
          dispatch(actions.setLoading(true))
          // add to trade
          const tradesPeople = await dispatch(
            TaskActions.addPeopleToTrade({
              taskId: 0,
              subs: [
                {
                  name: contact.get('name'),
                  cell: contact.get('phone')
                }
              ],
              projectId: project.get('id')
            })
          )
          const subsId = tradesPeople.data.subs
            .filter(sub => sub.cell === contact.get('phone'))
            .map(sub => sub.id)
          // add to conversation
          const conversationResponse = await dispatch(
            ConversationActions.createConversationWith({
              projectId: project.get('id'),
              partecipants: subsId
            })
          )
          await dispatch(
            ConversationActions.loadConversation({
              conversationId: conversationResponse.data.newConversationId
            })
          )
          await dispatch(
            InboxActions.fetchList({ projectId: project.get('id') })
          )
          const inbox = InboxSelectors.getConversation(
            conversationResponse.data.newConversationId
          )(getState())
          dispatch(actions.setLoading(false))
          // back + redirect
          dispatch(
            RouteActions.navigate(SCREENS.CONVERSATION, {
              conversationId: inbox.get('conversation'),
              roomId: inbox.get('room')
            })
          )
        } catch (error) {
          console.log(error)
          alert('Unable to create a message')
        }
      }
    }
  },
  toCall () {
    return async (dispatch, getState) => {
      const project = ProjectSelectors.getProject(projectId)(getState())
      const phoneNumber = project.get('phoneNumber')
      if (phoneNumber && phoneNumber.length > 1) {
        try {
          await comms.phonecall(phoneNumber, false)
        } catch (error) {
          alert('Unable to call')
        }
      }
    }
  },
  toDirections () {
    return (dispatch, getState) => {
      const project = ProjectSelectors.getProject(projectId)(getState())
      showLocation({
        latitude: project.get('lat'),
        longitude: project.get('long'),
        title: sanitizeLocation(project.get('location'))
      })
    }
  },
  toCaptureImages () {
    return (dispatch, getState) => {
      const projectId = selectors.projectId(getState())
      dispatch(RouteActions.navigate(SCREENS.PROJECT_IMAGE, { projectId }))
    }
  },
  toInbox () {
    return (dispatch, getState) => {
      const projectId = selectors.projectId(getState())
      dispatch(
        RouteActions.navigate(SCREENS.INBOX, {
          projectId,
          fromProjectInfo: true
        })
      )
    }
  },
  toRecord () {
    return (dispatch, getState) => {
      const projectId = selectors.projectId(getState())
      dispatch(RouteActions.navigate(SCREENS.PROJECT_AUDIO, { projectId }))
    }
  },
  toAttachFiles () {
    return (dispatch, getState) => {
      const projectId = selectors.projectId(getState())
      dispatch(RouteActions.navigate(SCREENS.PROJECT_FILE, { projectId }))
    }
  },
  toEditProject () {
    return (dispatch, getState) => {
      const projectId = selectors.projectId(getState())
      dispatch(RouteActions.navigate(SCREENS.PROJECT_EDIT, { projectId }))
    }
  },
  toConversation ({ conversationId, roomId, userId }) {
    return async (dispatch, getState) => {
      const projectId = selectors.projectId(getState())
      // always create conversation
      /*
      if (conversationId) {
        dispatch(
          RouteActions.navigate(SCREENS.CONVERSATION, {
            conversationId,
            roomId
          })
        )
        return
      } */
      try {
        const { data } = await dispatch(
          ConversationActions.createSubConversation({
            subUserId: userId,
            projectId
          })
        )
        console.log('f', data)
        dispatch(
          RouteActions.navigate(SCREENS.CONVERSATION, {
            conversationId: data.conversationId,
            roomId: data.roomId
          })
        )
      } catch (error) {
        console.log('ProjectInfo::toConversation', error)
      }
    }
  },
  addSub () {
    return (dispatch, getState) => {
      const projectId = selectors.projectId(getState())
      dispatch(
        RouteActions.navigate(SCREENS.ADD_CONTACT, {
          onSave: actions.addContact,
          projectId
        })
      )
    }
  },
  addContact ({ name, cell, email }) {
    return async (dispatch, getState) => {
      const projectId = selectors.projectId(getState())
      await dispatch(TaskActions.fetchForProject({ projectId }))
    }
  },
  setLoading (loading) {
    return {
      type: CONSTANTS.SET_LOADING,
      payload: loading
    }
  },
  announce () {
    return async (dispatch, getState) => {
      try {
        const message = await dispatch(
          ModalActions.confirm('Write a message to announce')
        )
        const projectId = selectors.projectId(getState())
        await dispatch(ProjectActions.broadcastMessage({ projectId, message }))
        Alert.alert('Nailit!', 'Announcement sent')
      } catch (err) {
        console.log(err)
      }
    }
  },
  onDelete (userId, taskId) {
    return async (dispatch, getState) => {
      const projectId = selectors.projectId(getState())
      dispatch(
        TaskActions.removePeopleFromTrade({
          taskId: taskId,
          subs: [{ userId }],
          projectId
        })
      )
      await dispatch(TaskActions.fetchForProject({ projectId }))
    }
  }
}

export const ProjectInfoState = Record({
  projectId: null,
  isLoading: false
})

export const reducer = createReducer(new ProjectInfoState(), {
  [CONSTANTS.SET_PROJECT_ID]: (state, { payload }) =>
    state.set('projectId', payload),
  [CONSTANTS.SET_LOADING]: (state, { payload }) =>
    state.set('isLoading', payload)
})

export const viewState = state => state.screens.projectinfo
export const viewSelector = createViewSelector(viewState)
export const projectId = viewSelector('projectId')
export const isLoading = viewSelector('isLoading')

export const hasLoaded = createSelector(
  [
    ProjectSelectors.hasProjectLoaded(projectId),
    TaskSelectors.hasProjectLoaded(projectId)
  ],
  (projectLoaded, tradesLoaded) => projectLoaded && tradesLoaded
)

export const tasksSubList = createSelector(
  [
    TaskSelectors.tasks,
    TaskSelectors.getProject(projectId),
    ProjectSelectors.getProject(projectId),
    UserSelectors.users,
    ContactSelectors.contacts
  ],
  (tasks, taskList, project, users, contacts) => {
    return (
      (tasks &&
        taskList &&
        taskList.reduce((list, taskId) => {
          const task = tasks.get(taskId)
          const subs = task.get('taskSub', new List())
          const subMap = subs
            .map(sub =>
              fromJS({
                user: sub.get('id'),
                task: taskId,
                projectTask: task.get('projectTask')
              })
            )
            .filter(sub => {
              const subUser = sub.get('user')
              const user =
                users.find(
                  user =>
                    user.get('name') ===
                    contacts.getIn([project.getIn(['contacts', 0]), 'name'])
                ) || new Map()
              return subUser !== user.get('id')
            })
          return list.concat(subMap)
        }, new List())) ||
      new List()
    )
  }
)

export const hasPhoneNumber = createSelector(
  [ProjectSelectors.getProject(projectId), ContactSelectors.contacts],
  (project, contacts) =>
    project &&
    (contacts.getIn([project.getIn(['contacts', 0]), 'phone'], '') || '')
      .length > 0
)

export const isSub = createSelector(
  [ProjectSelectors.getProject(projectId)],
  project => project && project.get('isSub', false)
)

export const selectors = {
  projectId,
  project: ProjectSelectors.getProject(projectId),
  contacts: ContactSelectors.contacts,
  hasLoaded,
  isLoading,
  hasPhoneNumber,
  tasks: TaskSelectors.tasks,
  users: UserSelectors.users,
  tasksSubList,
  isSub
}
