// @flow
import { List, Record, fromJS } from 'immutable'
import createReducer from 'utils/createreducer'
import createConstants from 'utils/createconstants'
import { createSelector } from 'reselect'
import { ProjectSelectors, ProjectActions } from 'resources/project'
import createViewSelector from 'utils/createviewselector'
import { RouteActions } from 'routes'

export const CONSTANTS = createConstants('SCREEN_SELECT_PROJECT', {
  TOGGLE_PROJECT: null,
  SET_SEARCH: null,
  SET_SAVE: null,
  RESET: null,
  SET_SINGLE_SELECTION: null,
  SET_DISPLAY_ALL: null,
  SET_SELECTION: null
})

export const actions = {
  initialize ({ save, displayAll, singleSelection, selected = [] }) {
    return async (dispatch, getState) => {
      dispatch(actions.reset())
      dispatch(actions.setSingleSelection({ singleSelection }))
      dispatch(actions.setSave(save))
      dispatch(actions.setDisplayAll(displayAll))
      dispatch(actions.setSelection(selected))
      dispatch(ProjectActions.fetchList())
      dispatch(
        RouteActions.setParams({
          save: () => dispatch(actions.saveProjects({ save }))
        })
      )
    }
  },

  reset (projectId) {
    return {
      type: CONSTANTS.RESET
    }
  },

  setSave (save) {
    return {
      type: CONSTANTS.SET_SAVE,
      payload: save
    }
  },

  setSelection (selection) {
    return {
      type: CONSTANTS.SET_SELECTION,
      payload: selection
    }
  },

  saveProjects ({ save }) {
    return (dispatch, getState) => {
      const state = getState()
      const selection = selectors.selection(state)
      const save = selectors.save(state)
      if (save) {
        dispatch(save(selection))
      }
      dispatch(RouteActions.back())
    }
  },

  setSearch (needle) {
    return {
      type: CONSTANTS.SET_SEARCH,
      payload: needle
    }
  },

  selectProject (projectId) {
    return (dispatch, getState) => {
      dispatch({
        type: CONSTANTS.TOGGLE_PROJECT,
        payload: projectId
      })
      const state = getState()
      const save = selectors.save(state)
      const selection = selectors.selection(state)
      const singleSelection = selectors.singleSelection(state)
      if (singleSelection && selection.size > 0) {
        dispatch(actions.saveProjects({ save }))
      }
    }
  },

  setDisplayAll (displayAll) {
    return {
      type: CONSTANTS.SET_DISPLAY_ALL,
      payload: displayAll
    }
  },
  setSingleSelection (singleSelection) {
    return {
      type: CONSTANTS.SET_SINGLE_SELECTION,
      payload: singleSelection
    }
  }
}

export const SelectProjectState = Record({
  selection: List(),
  save: null,
  search: '',
  singleSelection: false,
  displayAll: false
})

export const reducer = createReducer(new SelectProjectState(), {
  [CONSTANTS.RESET]: state => new SelectProjectState(),
  [CONSTANTS.SET_SEARCH]: (state, { payload }) => state.set('search', payload),
  [CONSTANTS.SET_SAVE]: (state, { payload }) => state.set('save', payload),
  [CONSTANTS.SET_SELECTION]: (state, { payload }) =>
    state.set('selection', fromJS(payload)),
  [CONSTANTS.SET_DISPLAY_ALL]: (state, { payload }) =>
    state.set('displayAll', payload),
  [CONSTANTS.SET_SINGLE_SELECTION]: (state, { payload }) =>
    state.set('singleSelection', payload),
  [CONSTANTS.TOGGLE_PROJECT]: (state, { payload }) => {
    const singleSelection = state.get('singleSelection')
    if (!singleSelection.size) {
      return state.set('selection', fromJS([payload]))
    }
    const selection = state.get('selection')
    if (selection.includes(payload)) {
      const newSelection = selection.filter(id => id !== payload)
      return state.set('selection', newSelection)
    }
    const newSelection = selection.push(payload)
    return state.set('selection', newSelection)
  }
})

export const viewState = state => state.screens.selectProject
export const viewSelector = createViewSelector(viewState)
export const selection = viewSelector('selection')
export const searchText = viewSelector('search', '')
export const singleSelection = viewSelector('singleSelection', true)
export const save = viewSelector('save', '')
export const displayAll = viewSelector('displayAll', false)
export const projectList = createSelector(
  [ProjectSelectors.projects, ProjectSelectors.projectsList, searchText],
  (projects, projectList, search) =>
    projectList.filter(projectId => {
      const projectName = projects.getIn([projectId, 'name'], '').toLowerCase()
      const matchProjectName = projectName.indexOf(search.toLowerCase()) !== -1
      return matchProjectName
    })
)

export const selectors = {
  selection,
  searchText,
  hasLoaded: ProjectSelectors.hasListLoaded,
  projects: ProjectSelectors.projects,
  projectList,
  singleSelection,
  save
}
