import React from 'react'
import { shallow } from 'enzyme'
import createStoreForTesting from 'test/createstore'
import { NavigationActions } from 'react-navigation'
import { AsyncStorage } from 'react-native'
import { UserActions, UserSchema } from 'resources/user'
import SCREENS from 'routes/screens'
import userInfoRecord from 'test/records/userinfo.json'
import { actions, selector } from '../module'
import { USERID_KEY } from '../../../config'
import SidemenuUser from '../components/SidemenuUser'
import Sidemenu from '../Sidemenu'

function renderHelper ({ ...props }) {
  const noop = () => {}
  const user = new UserSchema({ name: 'daniele' })
  return shallow(
    <Sidemenu
      currentUser={user}
      toAccountInfo={noop}
      toProjects={noop}
      toLogout={noop}
      toInbox={noop}
      toSupport={noop}
      toReferFriend={noop}
      {...props}
    />
  )
}

describe('Sidemenu Screen', () => {
  describe('module', () => {
    it('initialize should load from redux current logged in user name', async () => {
      const helper = createStoreForTesting()
      const userId = '771'
      helper.dispatch(UserActions.setCurrentUser(userId))
      helper.addResponse(userInfoRecord)
      await helper.dispatch(UserActions.fetchInfo({ userId }))
      const props = selector(helper.getState())
      expect(props.currentUser.id).toBe(userId)
    })
    describe('logout', () => {
      it('remove the user from state and redirect to login', async () => {
        const store = createStoreForTesting()
        await AsyncStorage.setItem(USERID_KEY, '112233')
        await store.dispatch(actions.toLogout())
        const userKey = await AsyncStorage.setItem(USERID_KEY, '112233')
        expect(userKey).toBe(null)
      })
      it('redirects to login', async () => {
        const helper = createStoreForTesting()
        await helper.dispatch(actions.toLogout())
        const action = helper.spyActions.findAction(NavigationActions.NAVIGATE)
        expect(action).toEqual({
          params: undefined,
          routeName: SCREENS.LOGIN,
          type: 'Navigation/NAVIGATE'
        })
      })
    })
    describe('links', () => {
      it('should navigate to projects', () => {
        const helper = createStoreForTesting()
        helper.dispatch(actions.toProjects())
        const action = helper.spyActions.findAction(NavigationActions.NAVIGATE)
        expect(action).toEqual({
          action: {
            actions: [
              {
                routeName: SCREENS.PROJECT_LIST,
                type: 'Navigation/NAVIGATE'
              }
            ],
            index: 0,
            type: 'Navigation/RESET'
          },
          routeName: 'SignedIn',
          type: 'Navigation/NAVIGATE'
        })
      })
      it('should navigate to all the links in the view', () => {
        const helper = createStoreForTesting()
        helper.dispatch(actions.toInbox())
        const action = helper.spyActions.findAction(NavigationActions.NAVIGATE)
        expect(action).toEqual({
          action: {
            actions: [
              { routeName: 'SCREEN_INBOX', type: 'Navigation/NAVIGATE' }
            ],
            index: 0,
            key: undefined,
            type: 'Navigation/RESET'
          },
          routeName: 'SignedIn',
          type: 'Navigation/NAVIGATE'
        })
      })
    })
  })
  describe('view', () => {
    function findByIcon (nodes, icon) {
      return nodes.findWhere(node => node.prop('icon') === icon)
    }

    it('should render the name of logged in user', () => {
      const currentUser = new UserSchema({ name: 'alex' })
      const node = renderHelper({ currentUser })
      const userMenu = node.find(SidemenuUser)
      expect(userMenu).toHaveLength(1)
      expect(userMenu.prop('user')).toBe(currentUser)
    })
    it('should render the logout link', () => {
      const node = findByIcon(renderHelper(), 'ios-log-out')
      expect(node).toHaveLength(1)
      expect(node.prop('label')).toBe('Logout')
    })
    it('should render My Account link', () => {
      const node = findByIcon(renderHelper(), 'ios-contact-outline')
      expect(node).toHaveLength(1)
      expect(node.prop('label')).toBe('My Account')
    })
    it('should render Project List link', () => {
      const node = findByIcon(renderHelper(), 'ios-clipboard-outline')
      expect(node).toHaveLength(1)
      expect(node.prop('label')).toBe('Projects')
    })
    it('should render Refer-a-friend link', () => {
      const node = findByIcon(renderHelper(), 'ios-share-outline')
      expect(node).toHaveLength(1)
      expect(node.prop('label')).toBe('Refer a Friend')
    })
    it('should render Support link', () => {
      const node = findByIcon(renderHelper(), 'ios-help-buoy-outline')
      expect(node).toHaveLength(1)
      expect(node.prop('label')).toBe('Support')
    })
  })
})
